@@ -51,5 +51,5 @@ enum options_client
   OPT_TRIGGERS,
   OPT_IGNORE_TABLE,OPT_INSERT_IGNORE,OPT_SHOW_WARNINGS,OPT_DROP_DATABASE,
   OPT_TZ_UTC, OPT_AUTO_CLOSE, OPT_SSL_VERIFY_SERVER_CERT,
-  OPT_DEBUG_INFO
+  OPT_DEBUG_INFO, OPT_ERROR_LOG_FILE
 };
