@@ -91,15 +91,14 @@ static bool stop_passed= 0;
   This is because the event will be created (alloced) in read_log_event()
   (which returns a pointer) in check_header().
 */
-Format_description_log_event* description_event; 
+Format_description_log_event* glob_description_event; 
 
 static int dump_local_log_entries(PRINT_EVENT_INFO *print_event_info,
                                   const char* logname);
 static int dump_remote_log_entries(PRINT_EVENT_INFO *print_event_info,
                                    const char* logname);
 static int dump_log_entries(const char* logname);
-static int dump_remote_file(NET* net, const char* fname);
-static void die(const char* fmt, ...);
+static void die(const char* fmt, ...)  __attribute__ ((__noreturn__));
 static MYSQL* safe_connect();
 
 
@@ -156,11 +155,7 @@ class Load_log_processor
 
 public:
   Load_log_processor() {}
-  ~Load_log_processor()
-  {
-    destroy();
-    delete_dynamic(&file_names);
-  }
+  ~Load_log_processor() {}
 
   int init()
   {
@@ -180,20 +175,22 @@ public:
       target_dir_name_len= strlen(target_dir_name);
     }
   void destroy()
+  {
+    File_name_record *ptr= (File_name_record *)file_names.buffer;
+    File_name_record *end= ptr + file_names.elements;
+    for (; ptr < end; ptr++)
     {
-      File_name_record *ptr= (File_name_record *)file_names.buffer;
-      File_name_record *end= ptr + file_names.elements;
-      for (; ptr<end; ptr++)
+      if (ptr->fname)
       {
-	if (ptr->fname)
-	{
-          my_free(ptr->fname, MYF(MY_WME));
-          delete ptr->event;
-          bzero((char *)ptr, sizeof(File_name_record));
-	}
+        my_free(ptr->fname, MYF(MY_WME));
+        delete ptr->event;
+        bzero((char *)ptr, sizeof(File_name_record));
       }
     }
 
+    delete_dynamic(&file_names);
+  }
+
   /*
     Obtain Create_file event for LOAD DATA statement by its file_id.
 
@@ -560,7 +557,7 @@ int process_event(PRINT_EVENT_INFO *print_event_info, Log_event *ev,
       ce->print(result_file, print_event_info, TRUE);
 
       // If this binlog is not 3.23 ; why this test??
-      if (description_event->binlog_version >= 3)
+      if (glob_description_event->binlog_version >= 3)
       {
 	if (load_processor.process(ce))
 	  break;				// Error
@@ -595,9 +592,9 @@ Create_file event for file_id: %u\n",exv->file_id);
       break;
     }
     case FORMAT_DESCRIPTION_EVENT:
-      delete description_event;
-      description_event= (Format_description_log_event*) ev;
-      print_event_info->common_header_len= description_event->common_header_len;
+      delete glob_description_event;
+      glob_description_event= (Format_description_log_event*) ev;
+      print_event_info->common_header_len= glob_description_event->common_header_len;
       ev->print(result_file, print_event_info);
       /*
         We don't want this event to be deleted now, so let's hide it (I
@@ -803,7 +800,7 @@ static void die(const char* fmt, ...)
 
 static void print_version()
 {
-  printf("%s Ver 3.1 for %s at %s\n", my_progname, SYSTEM_TYPE, MACHINE_TYPE);
+  printf("%s Ver 3.2 for %s at %s\n", my_progname, SYSTEM_TYPE, MACHINE_TYPE);
   NETWARE_SET_SCREEN_MODE(1);
 }
 
@@ -974,7 +971,7 @@ static int dump_log_entries(const char* logname)
   This is not as smart as check_header() (used for local log); it will not work
   for a binlog which mixes format. TODO: fix this.
 */
-static int check_master_version(MYSQL* mysql,
+static int check_master_version(MYSQL *mysql_arg,
                                 Format_description_log_event
                                 **description_event)
 {
@@ -982,26 +979,31 @@ static int check_master_version(MYSQL* mysql,
   MYSQL_ROW row;
   const char* version;
 
-  if (mysql_query(mysql, "SELECT VERSION()") ||
-      !(res = mysql_store_result(mysql)))
+  if (mysql_query(mysql_arg, "SELECT VERSION()") ||
+      !(res = mysql_store_result(mysql_arg)))
   {
+    /* purecov: begin inspected */
     char errmsg[256];
-    strmake(errmsg, mysql_error(mysql), sizeof(errmsg)-1);
-    mysql_close(mysql);
+    strmake(errmsg, mysql_error(mysql_arg), sizeof(errmsg)-1);
+    mysql_close(mysql_arg);
     die("Error checking master version: %s", errmsg);
+    /* purecov: end */
   }
   if (!(row = mysql_fetch_row(res)))
   {
+    /* purecov: begin inspected */
     mysql_free_result(res);
     mysql_close(mysql);
     die("Master returned no rows for SELECT VERSION()");
-    return 1;
+    /* purecov: end */
   }
   if (!(version = row[0]))
   {
+    /* purecov: begin inspected */
     mysql_free_result(res);
-    mysql_close(mysql);
+    mysql_close(mysql_arg);
     die("Master reported NULL for the version");
+    /* purecov: end */
   }
 
   switch (*version) {
@@ -1020,11 +1022,11 @@ static int check_master_version(MYSQL* mysql,
     *description_event= new Format_description_log_event(3);
     break;
   default:
-    sql_print_error("Master reported unrecognized MySQL version '%s'",
-		    version);
+    /* purecov: begin inspected */
     mysql_free_result(res);
-    mysql_close(mysql);
-    return 1;
+    mysql_close(mysql_arg);
+    die("Master reported unrecognized MySQL version '%s'", version);
+    /* purecov: end */
   }
   mysql_free_result(res);
   return 0;
@@ -1052,12 +1054,12 @@ static int dump_remote_log_entries(PRINT_EVENT_INFO *print_event_info,
   mysql= safe_connect();
   net= &mysql->net;
 
-  if (check_master_version(mysql, &description_event))
+  if (check_master_version(mysql, &glob_description_event))
   {
     fprintf(stderr, "Could not find server version");
     DBUG_RETURN(1);
   }
-  if (!description_event || !description_event->is_valid())
+  if (!glob_description_event || !glob_description_event->is_valid())
   {
     fprintf(stderr, "Invalid Format_description log event; \
 could be out of memory");
@@ -1103,11 +1105,11 @@ could be out of memory");
     }
     if (len < 8 && net->read_pos[0] == 254)
       break; // end of data
-    DBUG_PRINT("info",( "len: %lu, net->read_pos[5]: %d\n",
+    DBUG_PRINT("info",( "len: %lu  net->read_pos[5]: %d\n",
 			len, net->read_pos[5]));
     if (!(ev= Log_event::read_log_event((const char*) net->read_pos + 1 ,
                                         len - 1, &error_msg,
-                                        description_event)))
+                                        glob_description_event)))
     {
       fprintf(stderr, "Could not construct log event object\n");
       error= 1;
@@ -1115,7 +1117,7 @@ could be out of memory");
     }   
 
     Log_event_type type= ev->get_type_code();
-    if (description_event->binlog_version >= 3 ||
+    if (glob_description_event->binlog_version >= 3 ||
         (type != LOAD_EVENT && type != CREATE_FILE_EVENT))
     {
       /*
@@ -1158,6 +1160,18 @@ could be out of memory");
           len= 1; // fake Rotate, so don't increment old_off
         }
       }
+      else if (type == FORMAT_DESCRIPTION_EVENT)
+      {
+        /*
+          This could be an fake Format_description_log_event that server
+          (5.0+) automatically sends to a slave on connect, before sending
+          a first event at the requested position.  If this is the case,
+          don't increment old_off. Real Format_description_log_event always
+          starts from BIN_LOG_HEADER_SIZE position.
+        */
+        if (old_off != BIN_LOG_HEADER_SIZE)
+          len= 1;         // fake event, don't increment old_off
+      }
       if ((error= process_event(print_event_info, ev, old_off)))
       {
 	error= ((error < 0) ? 0 : 1);
@@ -1170,16 +1184,16 @@ could be out of memory");
       const char *old_fname= le->fname;
       uint old_len= le->fname_len;
       File file;
-      
+
       if ((file= load_processor.prepare_new_file_for_old_format(le,fname)) < 0)
       {
         error= 1;
         goto err;
       }
-      
+
       if ((error= process_event(print_event_info, ev, old_off)))
       {
- 	my_close(file,MYF(MY_WME));
+        my_close(file,MYF(MY_WME));
 	error= ((error < 0) ? 0 : 1);
         goto err;
       }
@@ -1192,8 +1206,8 @@ could be out of memory");
       }
     }
     /*
-      Let's adjust offset for remote log as for local log to produce 
-      similar text.
+      Let's adjust offset for remote log as for local log to produce
+      similar text and to have --stop-position to work identically.
     */
     old_off+= len-1;
   }
@@ -1321,7 +1335,7 @@ static int dump_local_log_entries(PRINT_EVENT_INFO *print_event_info,
       my_close(fd, MYF(MY_WME));
       return 1;
     }
-    check_header(file, &description_event);
+    check_header(file, &glob_description_event);
   }
   else // reading from stdin;
   {
@@ -1347,7 +1361,7 @@ static int dump_local_log_entries(PRINT_EVENT_INFO *print_event_info,
     if (init_io_cache(file, fileno(stdin), 0, READ_CACHE, (my_off_t) 0,
 		      0, MYF(MY_WME | MY_NABP | MY_DONT_CHECK_FILESIZE)))
       return 1;
-    check_header(file, &description_event);
+    check_header(file, &glob_description_event);
     if (start_position)
     {
       /* skip 'start_position' characters from stdin */
@@ -1365,7 +1379,7 @@ static int dump_local_log_entries(PRINT_EVENT_INFO *print_event_info,
     }
   }
 
-  if (!description_event || !description_event->is_valid())
+  if (!glob_description_event || !glob_description_event->is_valid())
     die("Invalid Format_description log event; could be out of memory");
 
   if (!start_position && my_b_read(file, tmp_buff, BIN_LOG_HEADER_SIZE))
@@ -1378,14 +1392,14 @@ static int dump_local_log_entries(PRINT_EVENT_INFO *print_event_info,
     char llbuff[21];
     my_off_t old_off = my_b_tell(file);
 
-    Log_event* ev = Log_event::read_log_event(file, description_event);
+    Log_event* ev = Log_event::read_log_event(file, glob_description_event);
     if (!ev)
     {
       /*
         if binlog wasn't closed properly ("in use" flag is set) don't complain
         about a corruption, but treat it as EOF and move to the next binlog.
       */
-      if (description_event->flags & LOG_EVENT_BINLOG_IN_USE_F)
+      if (glob_description_event->flags & LOG_EVENT_BINLOG_IN_USE_F)
         file->error= 0;
       else if (file->error)
       {
@@ -1410,7 +1424,7 @@ end:
   if (fd >= 0)
     my_close(fd, MYF(MY_WME));
   end_io_cache(file);
-  delete description_event;
+  delete glob_description_event;
   return error;
 }
 
@@ -1513,6 +1527,7 @@ int main(int argc, char** argv)
   cleanup();
   free_defaults(defaults_argv);
   my_free_open_file_info();
+  load_processor.destroy();
   /* We cannot free DBUG, it is used in global destructors after exit(). */
   my_end(MY_DONT_FREE_DBUG);
   exit(exit_value);
