@@ -68,7 +68,7 @@ int heap_write(HP_INFO *info, const byte *record)
 
 err:
   if (my_errno == HA_ERR_FOUND_DUPP_KEY)
-    DBUG_PRINT("info",("Duplicate key: %d", keydef - share->keydef));
+    DBUG_PRINT("info",("Duplicate key: %d", (int) (keydef - share->keydef)));
   info->errkey= keydef - share->keydef;
   /*
     We don't need to delete non-inserted key from rb-tree.  Also, if
@@ -105,7 +105,6 @@ int hp_rb_write_key(HP_INFO *info, HP_KEYDEF *keyinfo, const byte *record,
   heap_rb_param custom_arg;
   uint old_allocated;
 
-  info->last_pos= NULL; /* For heap_rnext/heap_rprev */
   custom_arg.keyseg= keyinfo->seg;
   custom_arg.key_length= hp_rb_make_key(keyinfo, info->recbuf, record, recpos);
   if (keyinfo->flag & HA_NOSAME)
