@@ -11,6 +11,7 @@ Created 10/10/1995 Heikki Tuuri
 #define srv0start_h
 
 #include "univ.i"
+#include "ut0byte.h"
 
 /*************************************************************************
 Normalizes a directory path for Windows: converts slashes to backslashes. */
@@ -52,6 +53,16 @@ srv_parse_log_group_home_dirs(
 					error */
 	char*	str,			/* in: character string */
 	char***	log_group_home_dirs);	/* out, own: log group home dirs */
+/*************************************************************************
+Adds a slash or a backslash to the end of a string if it is missing
+and the string is not empty. */
+
+char*
+srv_add_path_separator_if_needed(
+/*=============================*/
+			/* out: string which has the separator if the
+			string is not empty */
+	char*	str);	/* in: null-terminated character string */
 /********************************************************************
 Starts Innobase and creates a new database if database files
 are not found and the user wants. Server parameters are
@@ -63,11 +74,20 @@ innobase_start_or_create_for_mysql(void);
 				/* out: DB_SUCCESS or error code */
 /********************************************************************
 Shuts down the Innobase database. */
-
 int
 innobase_shutdown_for_mysql(void);
 /*=============================*/
 				/* out: DB_SUCCESS or error code */
+extern	dulint	srv_shutdown_lsn;
+extern	dulint	srv_start_lsn;
+
+#ifdef __NETWARE__
+void set_panic_flag_for_netware(void);
+#endif
+
+#ifdef HAVE_DARWIN_THREADS
+extern	ibool	srv_have_fullfsync;
+#endif
 
 extern  ulint   srv_sizeof_trx_t_in_ha_innodb_cc;
 
@@ -75,6 +95,8 @@ extern  ibool   srv_is_being_started;
 extern	ibool	srv_startup_is_before_trx_rollback_phase;
 extern	ibool	srv_is_being_shut_down;
 
+extern  ibool	srv_start_raw_disk_in_use;
+
 /* At a shutdown the value first climbs from 0 to SRV_SHUTDOWN_CLEANUP
 and then to SRV_SHUTDOWN_LAST_PHASE, and so on */
 
@@ -84,4 +106,7 @@ extern  ulint   srv_shutdown_state;
 #define SRV_SHUTDOWN_LAST_PHASE	   2
 #define SRV_SHUTDOWN_EXIT_THREADS  3
 
+/* Log 'spaces' have id's >= this */
+#define SRV_LOG_SPACE_FIRST_ID		0xFFFFFFF0UL
+
 #endif
