@@ -104,11 +104,20 @@ trx_rollback(
 /***********************************************************************
 Rollback or clean up transactions which have no user session. If the
 transaction already was committed, then we clean up a possible insert
-undo log. If the transaction was not yet committed, then we roll it back. */
+undo log. If the transaction was not yet committed, then we roll it back. 
+Note: this is done in a background thread. */
 
-void
-trx_rollback_or_clean_all_without_sess(void);
-/*========================================*/
+#ifndef __WIN__
+void*
+#else
+ulint
+#endif
+trx_rollback_or_clean_all_without_sess(
+/*===================================*/
+                        /* out: a dummy parameter */
+        void*   arg __attribute__((unused)));
+                        /* in: a dummy parameter required by
+                        os_thread_create */
 /********************************************************************
 Finishes a transaction rollback. */
 
@@ -193,7 +202,7 @@ trx_rollback_to_savepoint_for_mysql(
 						DB_NO_SAVEPOINT,
 						otherwise DB_SUCCESS */
 	trx_t*		trx,			/* in: transaction handle */
-	char*		savepoint_name,		/* in: savepoint name */
+	const char*	savepoint_name,		/* in: savepoint name */
 	ib_longlong*	mysql_binlog_cache_pos);/* out: the MySQL binlog cache
 						position corresponding to this
 						savepoint; MySQL needs this
@@ -211,11 +220,26 @@ trx_savepoint_for_mysql(
 /*====================*/
 						/* out: always DB_SUCCESS */
 	trx_t*		trx,			/* in: transaction handle */
-	char*		savepoint_name,		/* in: savepoint name */
+	const char*	savepoint_name,		/* in: savepoint name */
 	ib_longlong	binlog_cache_pos);	/* in: MySQL binlog cache
 						position corresponding to this
 						connection at the time of the
 						savepoint */
+						
+/***********************************************************************
+Releases a named savepoint. Savepoints which
+were set after this savepoint are deleted. */
+
+ulint
+trx_release_savepoint_for_mysql(
+/*================================*/
+						/* out: if no savepoint
+						of the name found then
+						DB_NO_SAVEPOINT,
+						otherwise DB_SUCCESS */
+	trx_t*		trx,			/* in: transaction handle */
+	const char*	savepoint_name);	/* in: savepoint name */
+
 /***********************************************************************
 Frees savepoint structs. */
 
