@@ -1242,9 +1242,10 @@ opt_print_query_plan(
 		}
 
 		fputs("Table ", stderr);
-		dict_index_name_print(stderr, plan->index);
+		dict_index_name_print(stderr, NULL, plan->index);
 		fprintf(stderr,"; exact m. %lu, match %lu, end conds %lu\n",
-			plan->n_exact_match, n_fields,
-			UT_LIST_GET_LEN(plan->end_conds));
+		        (unsigned long) plan->n_exact_match,
+		        (unsigned long) n_fields,
+			(unsigned long) UT_LIST_GET_LEN(plan->end_conds));
 	}
 }
