@@ -534,7 +534,7 @@ pars_retrieve_table_def(
 /*====================*/
 	sym_node_t*	sym_node)	/* in: table node */
 {
-	char*	table_name;
+	const char*	table_name;
 
 	ut_a(sym_node);
 	ut_a(que_node_get_type(sym_node) == QUE_NODE_SYMBOL);
@@ -542,7 +542,7 @@ pars_retrieve_table_def(
 	sym_node->resolved = TRUE;
 	sym_node->token_type = SYM_TABLE;
 
-	table_name = (char*) sym_node->name;
+	table_name = (const char*) sym_node->name;
 	
 	sym_node->table = dict_table_get_low(table_name);
 
@@ -890,7 +890,7 @@ pars_process_assign_list(
 		upd_field_set_field_no(upd_field,
 				dict_index_get_nth_col_pos(clust_index,
 							col_sym->col_no),
-								clust_index);
+							clust_index, NULL);
 		upd_field->exp = assign_node->val;
 
 		if (!dtype_is_fixed_size(
@@ -1514,8 +1514,11 @@ pars_create_table(
 
 	n_cols = que_node_list_get_len(column_defs);
 
-	table = dict_mem_table_create(table_sym->name, 0, n_cols);
-	
+	/* As the InnoDB SQL parser is for internal use only,
+	for creating some system tables, this function will only
+	create tables in the old (not compact) record format. */
+	table = dict_mem_table_create(table_sym->name, 0, n_cols, FALSE);
+
 	if (not_fit_in_memory != NULL) {
 		table->does_not_fit_in_memory = TRUE;
 	}
@@ -1717,7 +1720,8 @@ Called by yyparse on error. */
 void
 yyerror(
 /*====*/
-        char*	s __attribute__((unused))) /* in: error message string */
+	const char*	s __attribute__((unused)))
+				/* in: error message string */
 {
 	ut_ad(s);
 
@@ -1732,8 +1736,8 @@ Parses an SQL string returning the query graph. */
 que_t*
 pars_sql(
 /*=====*/
-			/* out, own: the query graph */
-	char*	str)	/* in: SQL string */
+				/* out, own: the query graph */
+	const char*	str)	/* in: SQL string */
 {
 	sym_node_t*	sym_node;
 	mem_heap_t*	heap;
