@@ -72,6 +72,24 @@ extern int check_definition(MI_KEYDEF *t1_keyinfo, MI_COLUMNDEF *t1_recinfo,
                             uint t1_keys, uint t1_recs,
                             MI_KEYDEF *t2_keyinfo, MI_COLUMNDEF *t2_recinfo,
                             uint t2_keys, uint t2_recs, bool strict);
+static void split_file_name(const char *file_name,
+			    LEX_STRING *db, LEX_STRING *name);
+
+
+extern "C" void myrg_print_wrong_table(const char *table_name)
+{
+  LEX_STRING db, name;
+  char buf[FN_REFLEN];
+  split_file_name(table_name, &db, &name);
+  memcpy(buf, db.str, db.length);
+  buf[db.length]= '.';
+  memcpy(buf + db.length + 1, name.str, name.length);
+  buf[db.length + name.length + 1]= 0;
+  push_warning_printf(current_thd, MYSQL_ERROR::WARN_LEVEL_ERROR,
+                      ER_ADMIN_WRONG_MRG_TABLE, ER(ER_ADMIN_WRONG_MRG_TABLE),
+                      buf);
+}
+
 
 const char **ha_myisammrg::bas_ext() const
 {
@@ -121,6 +139,8 @@ int ha_myisammrg::open(const char *name, int mode, uint test_if_locked)
   {
     DBUG_PRINT("error",("reclength: %lu  mean_rec_length: %lu",
 			table->s->reclength, mean_rec_length));
+    if (test_if_locked & HA_OPEN_FOR_REPAIR)
+      myrg_print_wrong_table(file->open_tables->table->filename);
     error= HA_ERR_WRONG_MRG_TABLE_DEF;
     goto err;
   }
@@ -139,12 +159,19 @@ int ha_myisammrg::open(const char *name, int mode, uint test_if_locked)
                          u_table->table->s->base.keys,
                          u_table->table->s->base.fields, false))
     {
-      my_free((gptr) recinfo, MYF(0));
       error= HA_ERR_WRONG_MRG_TABLE_DEF;
-      goto err;
+      if (test_if_locked & HA_OPEN_FOR_REPAIR)
+        myrg_print_wrong_table(u_table->table->filename);
+      else
+      {
+        my_free((gptr) recinfo, MYF(0));
+        goto err;
+      }
     }
   }
   my_free((gptr) recinfo, MYF(0));
+  if (error == HA_ERR_WRONG_MRG_TABLE_DEF)
+    goto err;
 #if !defined(BIG_TABLES) || SIZEOF_OFF_T == 4
   /* Merge table has more than 2G rows */
   if (table->s->crashed)
@@ -299,8 +326,8 @@ int ha_myisammrg::rnd_pos(byte * buf, byte *pos)
 
 void ha_myisammrg::position(const byte *record)
 {
-  ulonglong position= myrg_position(file);
-  my_store_ptr(ref, ref_length, (my_off_t) position);
+  ulonglong row_position= myrg_position(file);
+  my_store_ptr(ref, ref_length, (my_off_t) row_position);
 }
 
 
@@ -313,25 +340,25 @@ ha_rows ha_myisammrg::records_in_range(uint inx, key_range *min_key,
 
 int ha_myisammrg::info(uint flag)
 {
-  MYMERGE_INFO info;
-  (void) myrg_status(file,&info,flag);
+  MYMERGE_INFO mrg_info;
+  (void) myrg_status(file,&mrg_info,flag);
   /*
     The following fails if one has not compiled MySQL with -DBIG_TABLES
     and one has more than 2^32 rows in the merge tables.
   */
-  records = (ha_rows) info.records;
-  deleted = (ha_rows) info.deleted;
+  records = (ha_rows) mrg_info.records;
+  deleted = (ha_rows) mrg_info.deleted;
 #if !defined(BIG_TABLES) || SIZEOF_OFF_T == 4
-  if ((info.records >= (ulonglong) 1 << 32) ||
-      (info.deleted >= (ulonglong) 1 << 32))
+  if ((mrg_info.records >= (ulonglong) 1 << 32) ||
+      (mrg_info.deleted >= (ulonglong) 1 << 32))
     table->s->crashed= 1;
 #endif
-  data_file_length=info.data_file_length;
-  errkey  = info.errkey;
+  data_file_length=mrg_info.data_file_length;
+  errkey  = mrg_info.errkey;
   table->s->keys_in_use.set_prefix(table->s->keys);
-  table->s->db_options_in_use= info.options;
+  table->s->db_options_in_use= mrg_info.options;
   table->s->is_view= 1;
-  mean_rec_length= info.reclength;
+  mean_rec_length= mrg_info.reclength;
   
   /* 
     The handler::block_size is used all over the code in index scan cost
@@ -361,7 +388,7 @@ int ha_myisammrg::info(uint flag)
 #endif
   if (flag & HA_STATUS_CONST)
   {
-    if (table->s->key_parts && info.rec_per_key)
+    if (table->s->key_parts && mrg_info.rec_per_key)
     {
 #ifdef HAVE_purify
       /*
@@ -374,7 +401,7 @@ int ha_myisammrg::info(uint flag)
             sizeof(table->key_info[0].rec_per_key) * table->s->key_parts);
 #endif
       memcpy((char*) table->key_info[0].rec_per_key,
-	     (char*) info.rec_per_key,
+	     (char*) mrg_info.rec_per_key,
              sizeof(table->key_info[0].rec_per_key) *
              min(file->keys, table->s->key_parts));
     }
@@ -597,3 +624,9 @@ void ha_myisammrg::append_create_info(String *packet)
   }
   packet->append(')');
 }
+
+
+int ha_myisammrg::check(THD* thd, HA_CHECK_OPT* check_opt)
+{
+  return HA_ADMIN_OK;
+}
