@@ -21,8 +21,6 @@
 
 void unireg_init(ulong options)
 {
-  uint i;
-  double nr;
   DBUG_ENTER("unireg_init");
 
   MYSYS_PROGRAM_DONT_USE_CURSES();
@@ -39,16 +37,5 @@ void unireg_init(ulong options)
 
   VOID(strmov(reg_ext,".frm"));
   specialflag=SPECIAL_SAME_DB_NAME | options;  /* Set options from argv */
-  /* Make a tab of powers of 10 */
-  for (i=0,nr=1.0; i < array_elements(log_10) ; i++)
-  {					/* It's used by filesort... */
-    log_10[i]= nr ; nr*= 10.0;
-  }
-  /* Make a tab of powers of 0.1 */
-  for (i= 0, nr= 0.1; i < array_elements(log_01); i++)
-  {
-    log_01[i]= nr;
-    nr*= 0.1;
-  }
   DBUG_VOID_RETURN;
 }
