@@ -590,7 +590,7 @@ void *create_embedded_thd(int client_flag)
   thd->set_time();
   thd->init_for_queries();
   thd->client_capabilities= client_flag;
-  thd->real_id= (pthread_t) thd;
+  thd->real_id= (pthread_t)thd->thread_id;
 
   thd->db= NULL;
   thd->db_length= 0;
@@ -1052,7 +1052,7 @@ void Protocol_simple::prepare_for_resend()
   data->embedded_info->prev_ptr= &cur->next;
   next_field=cur->data;
   next_mysql_field= data->embedded_info->fields_list;
-err:
+
   DBUG_VOID_RETURN;
 }
 
