@@ -85,8 +85,7 @@ my_bool check_date(const MYSQL_TIME *ltime, my_bool not_zero_date,
         (!(flags & TIME_INVALID_DATES) &&
          ltime->month && ltime->day > days_in_month[ltime->month-1] &&
          (ltime->month != 2 || calc_days_in_year(ltime->year) != 366 ||
-          ltime->day != 29)) ||
-        (ltime->year == 0 && (ltime->month != 0 || ltime->day != 0)))
+          ltime->day != 29)))
     {
       *was_cut= 2;
       return TRUE;
@@ -206,7 +205,7 @@ str_to_datetime(const char *str, uint length, MYSQL_TIME *l_time,
   digits= (uint) (pos-str);
   start_loop= 0;                                /* Start of scan loop */
   date_len[format_position[0]]= 0;              /* Length of year field */
-  if (pos == end)
+  if (pos == end || *pos == '.')
   {
     /* Found date in internal format (only numbers like YYYYMMDD) */
     year_length= (digits == 4 || digits == 8 || digits >= 14) ? 4 : 2;
@@ -410,9 +409,7 @@ str_to_datetime(const char *str, uint length, MYSQL_TIME *l_time,
   if (number_of_fields < 3 ||
       l_time->year > 9999 || l_time->month > 12 ||
       l_time->day > 31 || l_time->hour > 23 ||
-      l_time->minute > 59 || l_time->second > 59 ||
-      (l_time->year == 0 && l_time->month == 0 && l_time->day == 0 && 
-       (l_time->hour != 0 || l_time->minute != 0 || l_time->second != 0)))
+      l_time->minute > 59 || l_time->second > 59)
   {
     /* Only give warning for a zero date if there is some garbage after */
     if (!not_zero_date)                         /* If zero date */
@@ -680,24 +677,24 @@ fractional:
     1        time value is invalid
 */
 
-int check_time_range(struct st_mysql_time *time, int *warning) 
+int check_time_range(struct st_mysql_time *my_time, int *warning) 
 {
   longlong hour;
 
-  if (time->minute >= 60 || time->second >= 60)
+  if (my_time->minute >= 60 || my_time->second >= 60)
     return 1;
 
-  hour= time->hour + (24*time->day);
+  hour= my_time->hour + (24*my_time->day);
   if (hour <= TIME_MAX_HOUR &&
-      (hour != TIME_MAX_HOUR || time->minute != TIME_MAX_MINUTE ||
-       time->second != TIME_MAX_SECOND || !time->second_part))
+      (hour != TIME_MAX_HOUR || my_time->minute != TIME_MAX_MINUTE ||
+       my_time->second != TIME_MAX_SECOND || !my_time->second_part))
     return 0;
 
-  time->day= 0;
-  time->hour= TIME_MAX_HOUR;
-  time->minute= TIME_MAX_MINUTE;
-  time->second= TIME_MAX_SECOND;
-  time->second_part= 0;
+  my_time->day= 0;
+  my_time->hour= TIME_MAX_HOUR;
+  my_time->minute= TIME_MAX_MINUTE;
+  my_time->second= TIME_MAX_SECOND;
+  my_time->second_part= 0;
   *warning|= MYSQL_TIME_WARN_OUT_OF_RANGE;
   return 0;
 }
@@ -730,7 +727,39 @@ void init_time(void)
 }
 
 
-	/* Calculate nr of day since year 0 in new date-system (from 1615) */
+/*
+  Handle 2 digit year conversions
+
+  SYNOPSIS
+  year_2000_handling()
+  year     2 digit year
+
+  RETURN
+    Year between 1970-2069
+*/
+
+uint year_2000_handling(uint year)
+{
+  if ((year=year+1900) < 1900+YY_PART_YEAR)
+    year+=100;
+  return year;
+}
+
+
+/*
+  Calculate nr of day since year 0 in new date-system (from 1615)
+
+  SYNOPSIS
+    calc_daynr()
+    year		 Year (exact 4 digit year, no year conversions)
+    month		 Month
+    day			 Day
+
+  NOTES: 0000-00-00 is a valid date, and will return 0
+
+  RETURN
+    Days since 0000-00-00
+*/
 
 long calc_daynr(uint year,uint month,uint day)
 {
@@ -1123,6 +1152,7 @@ longlong number_to_datetime(longlong nr, MYSQL_TIME *time_res,
  ok:
   part1=(long) (nr/LL(1000000));
   part2=(long) (nr - (longlong) part1*LL(1000000));
+  bzero((char*) time_res, sizeof(*time_res));
   time_res->year=  (int) (part1/10000L);  part1%=10000L;
   time_res->month= (int) part1 / 100;
   time_res->day=   (int) part1 % 100;
@@ -1148,22 +1178,23 @@ longlong number_to_datetime(longlong nr, MYSQL_TIME *time_res,
 
 /* Convert time value to integer in YYYYMMDDHHMMSS format */
 
-ulonglong TIME_to_ulonglong_datetime(const MYSQL_TIME *time)
+ulonglong TIME_to_ulonglong_datetime(const MYSQL_TIME *my_time)
 {
-  return ((ulonglong) (time->year * 10000UL +
-                       time->month * 100UL +
-                       time->day) * ULL(1000000) +
-          (ulonglong) (time->hour * 10000UL +
-                       time->minute * 100UL +
-                       time->second));
+  return ((ulonglong) (my_time->year * 10000UL +
+                       my_time->month * 100UL +
+                       my_time->day) * ULL(1000000) +
+          (ulonglong) (my_time->hour * 10000UL +
+                       my_time->minute * 100UL +
+                       my_time->second));
 }
 
 
 /* Convert TIME value to integer in YYYYMMDD format */
 
-ulonglong TIME_to_ulonglong_date(const MYSQL_TIME *time)
+ulonglong TIME_to_ulonglong_date(const MYSQL_TIME *my_time)
 {
-  return (ulonglong) (time->year * 10000UL + time->month * 100UL + time->day);
+  return (ulonglong) (my_time->year * 10000UL + my_time->month * 100UL +
+                      my_time->day);
 }
 
 
@@ -1173,11 +1204,11 @@ ulonglong TIME_to_ulonglong_date(const MYSQL_TIME *time)
   it's assumed that days have been converted to hours already.
 */
 
-ulonglong TIME_to_ulonglong_time(const MYSQL_TIME *time)
+ulonglong TIME_to_ulonglong_time(const MYSQL_TIME *my_time)
 {
-  return (ulonglong) (time->hour * 10000UL +
-                      time->minute * 100UL +
-                      time->second);
+  return (ulonglong) (my_time->hour * 10000UL +
+                      my_time->minute * 100UL +
+                      my_time->second);
 }
 
 
@@ -1201,15 +1232,15 @@ ulonglong TIME_to_ulonglong_time(const MYSQL_TIME *time)
     valid date either.
 */
 
-ulonglong TIME_to_ulonglong(const MYSQL_TIME *time)
+ulonglong TIME_to_ulonglong(const MYSQL_TIME *my_time)
 {
-  switch (time->time_type) {
+  switch (my_time->time_type) {
   case MYSQL_TIMESTAMP_DATETIME:
-    return TIME_to_ulonglong_datetime(time);
+    return TIME_to_ulonglong_datetime(my_time);
   case MYSQL_TIMESTAMP_DATE:
-    return TIME_to_ulonglong_date(time);
+    return TIME_to_ulonglong_date(my_time);
   case MYSQL_TIMESTAMP_TIME:
-    return TIME_to_ulonglong_time(time);
+    return TIME_to_ulonglong_time(my_time);
   case MYSQL_TIMESTAMP_NONE:
   case MYSQL_TIMESTAMP_ERROR:
     return ULL(0);
