@@ -273,7 +273,7 @@ db_find_routine(THD *thd, int type, sp_name *name, sp_head **sphp)
   if ((ret= db_find_routine_aux(thd, type, name, table)) != SP_OK)
     goto done;
 
-  if (table->s->fields != MYSQL_PROC_FIELD_COUNT)
+  if (table->s->fields < MYSQL_PROC_FIELD_COUNT)
   {
     ret= SP_GET_FIELD_FAILED;
     goto done;
@@ -434,21 +434,39 @@ db_load_routine(THD *thd, int type, sp_name *name, sp_head **sphp,
   if ((ret= sp_use_new_db(thd, name->m_db, &old_db, 1, &dbchanged)))
     goto end;
 
-  lex_start(thd, (uchar*)defstr.c_ptr(), defstr.length());
+  {
+    Lex_input_stream lip(thd, defstr.c_ptr(), defstr.length());
+    thd->m_lip= &lip;
+    lex_start(thd);
+    thd->spcont= NULL;
+    ret= MYSQLparse(thd);
 
-  thd->spcont= 0;
-  if (MYSQLparse(thd) || thd->is_fatal_error || newlex.sphead == NULL)
+    if (ret == 0)
+    {
+      /*
+        Not strictly necessary to invoke this method here, since we know
+        that we've parsed CREATE PROCEDURE/FUNCTION and not an
+        UPDATE/DELETE/INSERT/REPLACE/LOAD/CREATE TABLE, but we try to
+        maintain the invariant that this method is called for each
+        distinct statement, in case its logic is extended with other
+        types of analyses in future.
+      */
+      newlex.set_trg_event_type_for_tables();
+    }
+  }
+
+  if (ret || thd->is_fatal_error || newlex.sphead == NULL)
   {
     sp_head *sp= newlex.sphead;
 
-    if (dbchanged && (ret= mysql_change_db(thd, old_db.str, 1)))
+    if (dbchanged && (ret= mysql_change_db(thd, &old_db, TRUE)))
       goto end;
     delete sp;
     ret= SP_PARSE_ERROR;
   }
   else
   {
-    if (dbchanged && (ret= mysql_change_db(thd, old_db.str, 1)))
+    if (dbchanged && (ret= mysql_change_db(thd, &old_db, TRUE)))
       goto end;
     *sphp= newlex.sphead;
     (*sphp)->set_definer(&definer_user_name, &definer_host_name);
@@ -491,8 +509,6 @@ db_create_routine(THD *thd, int type, sp_head *sp)
   int ret;
   TABLE *table;
   char definer[USER_HOST_BUFF_SIZE];
-  char old_db_buf[NAME_LEN+1];
-  LEX_STRING old_db= { old_db_buf, sizeof(old_db_buf) };
   DBUG_ENTER("db_create_routine");
   DBUG_PRINT("enter", ("type: %d name: %.*s",type,sp->m_name.length,
                        sp->m_name.str));
@@ -507,7 +523,7 @@ db_create_routine(THD *thd, int type, sp_head *sp)
     strxmov(definer, thd->lex->definer->user.str, "@",
             thd->lex->definer->host.str, NullS);
 
-    if (table->s->fields != MYSQL_PROC_FIELD_COUNT)
+    if (table->s->fields < MYSQL_PROC_FIELD_COUNT)
     {
       ret= SP_GET_FIELD_FAILED;
       goto done;
@@ -754,7 +770,7 @@ print_field_values(THD *thd, TABLE *table,
 	switch (used_field->field_type) {
 	case MYSQL_TYPE_TIMESTAMP:
 	  {
-	    TIME tmp_time;
+	    MYSQL_TIME tmp_time;
 
 	    bzero((char *)&tmp_time, sizeof(tmp_time));
 	    ((Field_timestamp *) used_field->field)->get_time(&tmp_time);
@@ -1070,7 +1086,7 @@ sp_exist_routines(THD *thd, TABLE_LIST *routines, bool any, bool no_error)
     lex_name.length= strlen(routine->table_name);
     lex_db.str= thd->strmake(routine->db, lex_db.length);
     lex_name.str= thd->strmake(routine->table_name, lex_name.length);
-    name= new sp_name(lex_db, lex_name);
+    name= new sp_name(lex_db, lex_name, true);
     name->init_qname(thd);
     sp_object_found= sp_find_routine(thd, TYPE_ENUM_PROCEDURE, name,
                                      &thd->sp_proc_cache, FALSE) != NULL ||
@@ -1739,31 +1755,39 @@ sp_cache_routines_and_add_tables_for_triggers(THD *thd, LEX *lex,
                                               TABLE_LIST *table)
 {
   int ret= 0;
-  Table_triggers_list *triggers= table->table->triggers;
-  if (add_used_routine(lex, thd->stmt_arena, &triggers->sroutines_key,
-                       table->belong_to_view))
+
+  Sroutine_hash_entry **last_cached_routine_ptr=
+    (Sroutine_hash_entry **)lex->sroutines_list.next;
+
+  if (static_cast<int>(table->lock_type) >=
+      static_cast<int>(TL_WRITE_ALLOW_WRITE))
   {
-    Sroutine_hash_entry **last_cached_routine_ptr=
-                            (Sroutine_hash_entry **)lex->sroutines_list.next;
     for (int i= 0; i < (int)TRG_EVENT_MAX; i++)
     {
-      for (int j= 0; j < (int)TRG_ACTION_MAX; j++)
+      if (table->trg_event_map &
+          static_cast<uint8>(1 << static_cast<int>(i)))
       {
-        if (triggers->bodies[i][j])
+        for (int j= 0; j < (int)TRG_ACTION_MAX; j++)
         {
-          (void)triggers->bodies[i][j]->
-                add_used_tables_to_table_list(thd, &lex->query_tables_last,
-                                              table->belong_to_view);
-          sp_update_stmt_used_routines(thd, lex,
-                                       &triggers->bodies[i][j]->m_sroutines,
-                                       table->belong_to_view);
+          /* We can have only one trigger per action type currently */
+          sp_head *trigger= table->table->triggers->bodies[i][j];
+          if (trigger &&
+              add_used_routine(lex, thd->stmt_arena, &trigger->m_sroutines_key,
+                               table->belong_to_view))
+          {
+            trigger->add_used_tables_to_table_list(thd, &lex->query_tables_last,
+                                                   table->belong_to_view);
+            sp_update_stmt_used_routines(thd, lex,
+                                         &trigger->m_sroutines,
+                                         table->belong_to_view);
+          }
         }
       }
     }
-    ret= sp_cache_routines_and_add_tables_aux(thd, lex,
-                                              *last_cached_routine_ptr, 
-                                              FALSE, NULL);
   }
+  ret= sp_cache_routines_and_add_tables_aux(thd, lex,
+                                            *last_cached_routine_ptr,
+                                            FALSE, NULL);
   return ret;
 }
 
@@ -1842,9 +1866,7 @@ create_string(THD *thd, String *buf,
   SYNOPSIS
     sp_use_new_db()
       thd            thread handle
-
       new_db         new database name (a string and its length)
-
       old_db         [IN] str points to a buffer where to store the old
                           database, length contains the size of the buffer
                      [OUT] if old db was not NULL, its name is copied
@@ -1852,7 +1874,6 @@ create_string(THD *thd, String *buf,
                      accordingly. Otherwise str[0] is set to '\0' and length
                      is set to 0. The out parameter should be used only if
                      the database name has been changed (see dbchangedp).
-
      dbchangedp      [OUT] is set to TRUE if the current database is changed,
                      FALSE otherwise. A database is not changed if the old
                      name is the same as the new one, both names are empty,
@@ -1873,15 +1894,11 @@ sp_use_new_db(THD *thd, LEX_STRING new_db, LEX_STRING *old_db,
   DBUG_PRINT("enter", ("newdb: %s", new_db.str));
 
   /*
-    Set new_db to an empty string if it's NULL, because mysql_change_db
-    requires a non-NULL argument.
-    new_db.str can be NULL only if we're restoring the old database after
-    execution of a stored procedure and there were no current database
-    selected. The stored procedure itself must always have its database
-    initialized.
+    A stored routine always belongs to some database. The
+    old database (old_db) might be NULL, but to restore the
+    old database we will use mysql_change_db.
   */
-  if (new_db.str == NULL)
-    new_db.str= empty_c_string;
+  DBUG_ASSERT(new_db.str && new_db.length);
 
   if (thd->db)
   {
@@ -1901,7 +1918,7 @@ sp_use_new_db(THD *thd, LEX_STRING new_db, LEX_STRING *old_db,
     DBUG_RETURN(0);
   }
 
-  ret= mysql_change_db(thd, new_db.str, no_access_check);
+  ret= mysql_change_db(thd, &new_db, no_access_check);
 
   *dbchangedp= ret == 0;
   DBUG_RETURN(ret);
