@@ -1,9 +1,8 @@
-/* Copyright (C) 2001 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+/* Copyright (C) 2001-2002, 2004 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,9 +21,9 @@ int myrg_write(register MYRG_INFO *info, byte *rec)
 {
   /* [phi] MERGE_WRITE_DISABLED is handled by the else case */
   if (info->merge_insert_method == MERGE_INSERT_TO_FIRST)
-    return mi_write(info->open_tables[0].table,rec);
+    return mi_write((info->current_table=info->open_tables)->table,rec);
   else if (info->merge_insert_method == MERGE_INSERT_TO_LAST)
-    return mi_write(info->end_table[-1].table,rec);
+    return mi_write((info->current_table=info->end_table-1)->table,rec);
   else /* unsupported insertion method */
     return (my_errno= HA_ERR_WRONG_COMMAND);
 }
