@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,6 +31,7 @@ File my_create_with_symlink(const char *linkname, const char *filename,
   int tmp_errno;
   /* Test if we should create a link */
   int create_link;
+  char abs_linkname[FN_REFLEN];
   DBUG_ENTER("my_create_with_symlink");
 
   if (my_disable_symlinks)
@@ -42,17 +42,23 @@ File my_create_with_symlink(const char *linkname, const char *filename,
       filename= linkname;
   }
   else
-    create_link= (linkname && strcmp(linkname,filename));    
+  {
+    if (linkname)
+      my_realpath(abs_linkname, linkname, MYF(0));
+    create_link= (linkname && strcmp(abs_linkname,filename));
+  }
 
   if (!(MyFlags & MY_DELETE_OLD))
   {
     if (!access(filename,F_OK))
     {
+      my_errno= errno= EEXIST;
       my_error(EE_CANTCREATEFILE, MYF(0), filename, EEXIST);
       DBUG_RETURN(-1);
     }
     if (create_link && !access(linkname,F_OK))
     {
+      my_errno= errno= EEXIST;
       my_error(EE_CANTCREATEFILE, MYF(0), linkname, EEXIST);
       DBUG_RETURN(-1);
     }
@@ -120,6 +126,7 @@ int my_rename_with_symlink(const char *from, const char *to, myf MyFlags)
   int was_symlink= (!my_disable_symlinks &&
 		    !my_readlink(link_name, from, MYF(0)));
   int result=0;
+  int name_is_different;
   DBUG_ENTER("my_rename_with_symlink");
 
   if (!was_symlink)
@@ -128,6 +135,14 @@ int my_rename_with_symlink(const char *from, const char *to, myf MyFlags)
   /* Change filename that symlink pointed to */
   strmov(tmp_name, to);
   fn_same(tmp_name,link_name,1);		/* Copy dir */
+  name_is_different= strcmp(link_name, tmp_name);
+  if (name_is_different && !access(tmp_name, F_OK))
+  {
+    my_errno= EEXIST;
+    if (MyFlags & MY_WME)
+      my_error(EE_CANTCREATEFILE, MYF(0), tmp_name, EEXIST);
+    DBUG_RETURN(1);
+  }
 
   /* Create new symlink */
   if (my_symlink(tmp_name, to, MyFlags))
@@ -139,7 +154,7 @@ int my_rename_with_symlink(const char *from, const char *to, myf MyFlags)
     the same basename and different directories.
    */
 
-  if (strcmp(link_name, tmp_name) && my_rename(link_name, tmp_name, MyFlags))
+  if (name_is_different && my_rename(link_name, tmp_name, MyFlags))
   {
     int save_errno=my_errno;
     my_delete(to, MyFlags);			/* Remove created symlink */
