@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,6 +20,10 @@
 #include "system_wrappers.h"
 #endif
 
+#ifdef HAVE_GETRUSAGE
+#include <sys/resource.h>
+#endif
+
 #ifdef THREAD
 #include <my_pthread.h>
 extern pthread_mutex_t THR_LOCK_malloc, THR_LOCK_open, THR_LOCK_keycache;
@@ -29,3 +32,11 @@ extern pthread_mutex_t THR_LOCK_charset;
 #else
 #include <my_no_pthread.h>
 #endif
+
+/*
+  EDQUOT is used only in 3 C files only in mysys/. If it does not exist on
+  system, we set it to some value which can never happen.
+*/
+#ifndef EDQUOT
+#define EDQUOT (-1)
+#endif
