@@ -67,14 +67,14 @@ TCP_Transporter::TCP_Transporter(TransporterRegistry &t_reg,
                                  const char *lHostName,
                                  const char *rHostName, 
                                  int r_port,
-				 bool isMgmConnection,
+				 bool isMgmConnection_arg,
 				 NodeId lNodeId,
                                  NodeId rNodeId,
 				 NodeId serverNodeId,
                                  bool chksm, bool signalId,
                                  Uint32 _reportFreq) :
   Transporter(t_reg, tt_TCP_TRANSPORTER,
-	      lHostName, rHostName, r_port, isMgmConnection,
+	      lHostName, rHostName, r_port, isMgmConnection_arg,
 	      lNodeId, rNodeId, serverNodeId,
 	      0, false, chksm, signalId),
   m_sendBuffer(sendBufSize)
@@ -155,6 +155,8 @@ TCP_Transporter::initTransporter() {
 
 void
 TCP_Transporter::setSocketOptions(){
+  int sockOptKeepAlive  = 1;
+
   if (setsockopt(theSocket, SOL_SOCKET, SO_RCVBUF,
                  (char*)&sockOptRcvBufSize, sizeof(sockOptRcvBufSize)) < 0) {
 #ifdef DEBUG_TRANSPORTER
@@ -169,6 +171,11 @@ TCP_Transporter::setSocketOptions(){
 #endif
   }//if
   
+  if (setsockopt(theSocket, SOL_SOCKET, SO_KEEPALIVE,
+                 (char*)&sockOptKeepAlive, sizeof(sockOptKeepAlive)) < 0) {
+    ndbout_c("The setsockopt SO_KEEPALIVE error code = %d", InetErrno);
+  }//if
+
   //-----------------------------------------------
   // Set the TCP_NODELAY option so also small packets are sent
   // as soon as possible
