@@ -67,7 +67,7 @@ public:
   ~Listener_thread();
   void run();
 private:
-  static const int LISTEN_BACK_LOG_SIZE= 5;     /* standard backlog size */
+  static const int LISTEN_BACK_LOG_SIZE;
   ulong total_connection_count;
   Thread_info thread_info;
 
@@ -80,6 +80,8 @@ private:
   int   create_unix_socket(struct sockaddr_un &unix_socket_address);
 };
 
+const int Listener_thread::LISTEN_BACK_LOG_SIZE = 5; /* standard backlog size */
+
 
 Listener_thread::Listener_thread(const Listener_thread_args &args) :
   Listener_thread_args(args.thread_registry, args.options, args.user_map,
@@ -210,10 +212,13 @@ void Listener_thread::run()
   return;
 
 err:
+  log_error("Listener: failed to initialize. Initiate shutdown...");
+
   // we have to close the ip sockets in case of error
   for (i= 0; i < num_sockets; i++)
     closesocket(sockets[i]);
 
+  thread_registry.set_error_status();
   thread_registry.unregister_thread(&thread_info);
   thread_registry.request_shutdown();
   my_thread_end();
