@@ -147,11 +147,11 @@ public:
   void count_only_length();
   void count_real_length();
   void count_decimal_length();
-  inline bool get_arg0_date(TIME *ltime, uint fuzzy_date)
+  inline bool get_arg0_date(MYSQL_TIME *ltime, uint fuzzy_date)
   {
     return (null_value=args[0]->get_date(ltime, fuzzy_date));
   }
-  inline bool get_arg0_time(TIME *ltime)
+  inline bool get_arg0_time(MYSQL_TIME *ltime)
   {
     return (null_value=args[0]->get_time(ltime));
   }
@@ -278,7 +278,7 @@ public:
   { max_length= 21; }
   Item_int_func(List<Item> &list) :Item_func(list) { max_length= 21; }
   Item_int_func(THD *thd, Item_int_func *item) :Item_func(thd, item) {}
-  double val_real() { DBUG_ASSERT(fixed == 1); return (double) val_int(); }
+  double val_real();
   String *val_str(String*str);
   enum Item_result result_type () const { return INT_RESULT; }
   void fix_length_and_dec() {}
@@ -303,12 +303,6 @@ class Item_func_signed :public Item_int_func
 public:
   Item_func_signed(Item *a) :Item_int_func(a) {}
   const char *func_name() const { return "cast_as_signed"; }
-  double val_real()
-  {
-    double tmp= args[0]->val_real();
-    null_value= args[0]->null_value;
-    return tmp;
-  }
   longlong val_int();
   longlong val_int_from_str(int *error);
   void fix_length_and_dec()
@@ -337,8 +331,8 @@ class Item_decimal_typecast :public Item_func
 public:
   Item_decimal_typecast(Item *a, int len, int dec) :Item_func(a)
   {
-    max_length= len + 2;
     decimals= dec;
+    max_length= my_decimal_precision_to_length(len, dec, unsigned_flag);
   }
   String *val_str(String *str);
   double val_real();
@@ -693,15 +687,24 @@ class Item_func_min_max :public Item_func
   Item_result cmp_type;
   String tmp_value;
   int cmp_sign;
+  /* TRUE <=> arguments should be compared in the DATETIME context. */
+  bool compare_as_dates;
+  /* An item used for issuing warnings while string to DATETIME conversion. */
+  Item *datetime_item;
+  THD *thd;
+
 public:
   Item_func_min_max(List<Item> &list,int cmp_sign_arg) :Item_func(list),
-    cmp_type(INT_RESULT), cmp_sign(cmp_sign_arg) {}
+    cmp_type(INT_RESULT), cmp_sign(cmp_sign_arg), compare_as_dates(FALSE),
+    datetime_item(0) {}
   double val_real();
   longlong val_int();
   String *val_str(String *);
   my_decimal *val_decimal(my_decimal *);
   void fix_length_and_dec();
   enum Item_result result_type () const { return cmp_type; }
+  bool result_as_longlong() { return compare_as_dates; };
+  uint cmp_datetimes(ulonglong *value);
 };
 
 class Item_func_min :public Item_func_min_max
@@ -719,6 +722,35 @@ public:
 };
 
 
+/* 
+  Objects of this class are used for ROLLUP queries to wrap up 
+  each constant item referred to in GROUP BY list. 
+*/
+
+class Item_func_rollup_const :public Item_func
+{
+public:
+  Item_func_rollup_const(Item *a) :Item_func(a)
+  {
+    name= a->name;
+    name_length= a->name_length;
+  }
+  double val_real() { return args[0]->val_real(); }
+  longlong val_int() { return args[0]->val_int(); }
+  String *val_str(String *str) { return args[0]->val_str(str); }
+  my_decimal *val_decimal(my_decimal *dec) { return args[0]->val_decimal(dec); }
+  const char *func_name() const { return "rollup_const"; }
+  bool const_item() const { return 0; }
+  Item_result result_type() const { return args[0]->result_type(); }
+  void fix_length_and_dec()
+  {
+    collation= args[0]->collation;
+    max_length= args[0]->max_length;
+    decimals=args[0]->decimals; 
+  }
+};
+
+
 class Item_func_length :public Item_int_func
 {
   String value;
@@ -1176,6 +1208,10 @@ public:
   longlong val_int();
   String *val_str(String *str);
   my_decimal *val_decimal(my_decimal *);
+  double val_result();
+  longlong val_int_result();
+  String *str_result(String *str);
+  my_decimal *val_decimal_result(my_decimal *);
   bool update_hash(void *ptr, uint length, enum Item_result type,
   		   CHARSET_INFO *cs, Derivation dv, bool unsigned_arg);
   bool send(Protocol *protocol, String *str_arg);
@@ -1188,7 +1224,13 @@ public:
   void print(String *str);
   void print_as_stmt(String *str);
   const char *func_name() const { return "set_user_var"; }
-  int save_in_field(Field *field, bool no_conversions);
+  int save_in_field(Field *field, bool no_conversions,
+                    bool can_use_result_field);
+  int save_in_field(Field *field, bool no_conversions)
+  {
+    return save_in_field(field, no_conversions, 1);
+  }
+  void save_org_in_field(Field *field) { (void)save_in_field(field, 1, 0); }
 };
 
 
@@ -1308,12 +1350,12 @@ public:
   FT_INFO *ft_handler;
   TABLE *table;
   Item_func_match *master;   // for master-slave optimization
-  Item *concat;              // Item_func_concat_ws
-  String value;              // value of concat
+  Item *concat_ws;           // Item_func_concat_ws
+  String value;              // value of concat_ws
   String search_value;       // key_item()'s value converted to cmp_collation
 
   Item_func_match(List<Item> &a, uint b): Item_real_func(a), key(0), flags(b),
-       join_key(0), ft_handler(0), table(0), master(0), concat(0) { }
+       join_key(0), ft_handler(0), table(0), master(0), concat_ws(0) { }
   void cleanup()
   {
     DBUG_ENTER("Item_func_match");
@@ -1321,7 +1363,7 @@ public:
     if (!master && ft_handler)
       ft_handler->please->close_search(ft_handler);
     ft_handler= 0;
-    concat= 0;
+    concat_ws= 0;
     DBUG_VOID_RETURN;
   }
   enum Functype functype() const { return FT_FUNC; }
@@ -1405,13 +1447,16 @@ private:
   sp_name *m_name;
   mutable sp_head *m_sp;
   TABLE *dummy_table;
-  Field *result_field;
   char result_buf[64];
+  /*
+     The result field of the concrete stored function.
+  */
+  Field *sp_result_field;
 
-  bool execute(Field **flp);
-  bool execute_impl(THD *thd, Field *return_value_fld);
-  Field *sp_result_field(void) const;
-
+  bool execute();
+  bool execute_impl(THD *thd);
+  bool init_result_field(THD *thd);
+  
 public:
 
   Item_func_sp(Name_resolution_context *context_arg, sp_name *name);
@@ -1422,6 +1467,8 @@ public:
   virtual ~Item_func_sp()
   {}
 
+  void update_used_tables();
+
   void cleanup();
 
   const char *func_name() const;
@@ -1436,23 +1483,23 @@ public:
 
   longlong val_int()
   {
-    if (execute(&result_field))
+    if (execute())
       return (longlong) 0;
-    return result_field->val_int();
+    return sp_result_field->val_int();
   }
 
   double val_real()
   {
-    if (execute(&result_field))
+    if (execute())
       return 0.0;
-    return result_field->val_real();
+    return sp_result_field->val_real();
   }
 
   my_decimal *val_decimal(my_decimal *dec_buf)
   {
-    if (execute(&result_field))
+    if (execute())
       return NULL;
-    return result_field->val_decimal(dec_buf);
+    return sp_result_field->val_decimal(dec_buf);
   }
 
   String *val_str(String *str)
@@ -1461,7 +1508,7 @@ public:
     char buff[20];
     buf.set(buff, 20, str->charset());
     buf.length(0);
-    if (execute(&result_field))
+    if (execute())
       return NULL;
     /*
       result_field will set buf pointing to internal buffer
@@ -1469,7 +1516,7 @@ public:
       when SP is executed. In order to prevent occasional
       corruption of returned value, we make here a copy.
     */
-    result_field->val_str(&buf);
+    sp_result_field->val_str(&buf);
     str->copy(buf);
     return str;
   }
@@ -1477,11 +1524,11 @@ public:
   virtual bool change_context_processor(byte *cntx)
     { context= (Name_resolution_context *)cntx; return FALSE; }
 
-  void fix_length_and_dec();
-  bool find_and_check_access(THD * thd);
+  bool sp_check_access(THD * thd);
   virtual enum Functype functype() const { return FUNC_SP; }
 
   bool fix_fields(THD *thd, Item **ref);
+  void fix_length_and_dec(void);
   bool is_expensive() { return 1; }
 };
 
