@@ -42,8 +42,9 @@ class Table_triggers_list: public Sql_alloc
   */
   Field             **new_field;
   Field             **old_field;
+
   /* TABLE instance for which this triggers list object was created */
-  TABLE *table;
+  TABLE *trigger_table;
   /*
     Names of triggers.
     Should correspond to order of triggers on definitions_list,
@@ -55,14 +56,6 @@ class Table_triggers_list: public Sql_alloc
     updating trigger definitions during RENAME TABLE.
   */
   List<LEX_STRING>  on_table_names_list;
-  /*
-    Key representing triggers for this table in set of all stored
-    routines used by statement.
-    TODO: We won't need this member once triggers namespace will be
-    database-wide instead of table-wide because then we will be able
-    to use key based on sp_name as for other stored routines.
-  */
-  LEX_STRING        sroutines_key;
 
   /*
     Grant information for each trigger (pair: subject table, trigger definer).
@@ -83,7 +76,7 @@ public:
   List<LEX_STRING>  definers_list;
 
   Table_triggers_list(TABLE *table_arg):
-    record1_field(0), table(table_arg)
+    record1_field(0), trigger_table(table_arg)
   {
     bzero((char *)bodies, sizeof(bodies));
     bzero((char *)trigger_fields, sizeof(trigger_fields));
@@ -109,6 +102,11 @@ public:
                                 const char *old_table,
                                 const char *new_db,
                                 const char *new_table);
+  bool has_triggers(trg_event_type event_type, 
+                    trg_action_time_type action_time)
+  {
+    return (bodies[event_type][action_time] != NULL);
+  }
   bool has_delete_triggers()
   {
     return (bodies[TRG_EVENT_DELETE][TRG_ACTION_BEFORE] ||
@@ -138,3 +136,7 @@ private:
 
 extern const LEX_STRING trg_action_time_type_names[];
 extern const LEX_STRING trg_event_type_names[];
+
+int
+add_table_for_trigger(THD *thd, sp_name *trig, bool if_exists,
+                      TABLE_LIST **table);
