@@ -1,9 +1,8 @@
-/* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
+/* Copyright (C) 2000-2006 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +19,7 @@
 #include "mysql_priv.h"
 #include <errno.h>
 #include <m_ctype.h>
-#include "md5.h"
+#include "my_md5.h"
 
 	/* Functions defined in this file */
 
@@ -88,7 +87,7 @@ int openfrm(THD *thd, const char *name, const char *alias, uint db_stat,
   MEM_ROOT **root_ptr, *old_root;
   TABLE_SHARE *share;
   DBUG_ENTER("openfrm");
-  DBUG_PRINT("enter",("name: '%s'  form: 0x%lx",name,outparam));
+  DBUG_PRINT("enter",("name: '%s'  form: 0x%lx", name, (long) outparam));
 
   error= 1;
   disk_buff= NULL;
@@ -121,6 +120,8 @@ int openfrm(THD *thd, const char *name, const char *alias, uint db_stat,
     // caller can't process new .frm
     goto err;
   }
+  if (prgflag & OPEN_VIEW_NO_PARSE)
+    goto err;
 
   share->blob_ptr_size= sizeof(char*);
   outparam->db_stat= db_stat;
@@ -410,7 +411,9 @@ int openfrm(THD *thd, const char *name, const char *alias, uint db_stat,
   int_length= uint2korr(head+274);
   share->null_fields= uint2korr(head+282);
   com_length= uint2korr(head+284);
-  share->comment= strdup_root(&outparam->mem_root, (char*) head+47);
+  share->comment.length=  (int) (head[46]);
+  share->comment.str= strmake_root(&outparam->mem_root, (char*) head+47,
+                                   share->comment.length);
 
   DBUG_PRINT("info",("i_count: %d  i_parts: %d  index: %d  n_length: %d  int_length: %d  com_length: %d", interval_count,interval_parts, share->keys,n_length,int_length, com_length));
 
@@ -449,6 +452,8 @@ int openfrm(THD *thd, const char *name, const char *alias, uint db_stat,
   memcpy(comment_pos, disk_buff+read_length-com_length, com_length);
 
   fix_type_pointers(&int_array, &share->fieldnames, 1, &names);
+  if (share->fieldnames.count != share->fields)
+    goto err;
   fix_type_pointers(&int_array, share->intervals, interval_count,
 		    &names);
 
@@ -464,7 +469,10 @@ int openfrm(THD *thd, const char *name, const char *alias, uint db_stat,
                                                         count)))
         goto err;
       for (count= 0; count < interval->count; count++)
-        interval->type_lengths[count]= strlen(interval->type_names[count]);
+      {
+        char *val= (char*) interval->type_names[count];
+        interval->type_lengths[count]= strlen(val);
+      }
       interval->type_lengths[count]= 0;
     }
   }
@@ -712,6 +720,7 @@ int openfrm(THD *thd, const char *name, const char *alias, uint db_stat,
 	if (key_part->fieldnr)
 	{					// Should always be true !
 	  Field *field=key_part->field=outparam->field[key_part->fieldnr-1];
+          key_part->type= field->key_type();
 	  if (field->null_ptr)
 	  {
 	    key_part->null_offset=(uint) ((byte*) field->null_ptr -
@@ -771,7 +780,11 @@ int openfrm(THD *thd, const char *name, const char *alias, uint db_stat,
 	      the primary key, then we can use any key to find this column
 	    */
 	    if (ha_option & HA_PRIMARY_KEY_IN_READ_INDEX)
+            {
 	      field->part_of_key= share->keys_in_use;
+              if (field->part_of_sortkey.is_set(key))
+                field->part_of_sortkey= share->keys_in_use;
+            }
 	  }
 	  if (field->key_length() != key_part->length)
 	  {
@@ -802,7 +815,8 @@ int openfrm(THD *thd, const char *name, const char *alias, uint db_stat,
 	    if (!(field->flags & BLOB_FLAG))
 	    {					// Create a new field
 	      field=key_part->field=field->new_field(&outparam->mem_root,
-						     outparam);
+                                                     outparam,
+                                                     outparam == field->table);
 	      field->field_length=key_part->length;
 	    }
 	  }
@@ -815,7 +829,7 @@ int openfrm(THD *thd, const char *name, const char *alias, uint db_stat,
 	    as we need to test for NULL = NULL.
 	  */
 	  if (field->real_maybe_null())
-	    key_part->key_part_flag|= HA_PART_KEY_SEG;
+	    key_part->key_part_flag|= HA_NULL_PART;
 	}
 	else
 	{					// Error: shorten key
@@ -882,17 +896,17 @@ int openfrm(THD *thd, const char *name, const char *alias, uint db_stat,
   if (share->blob_fields)
   {
     Field **ptr;
-    uint i, *save;
+    uint k, *save;
 
     /* Store offsets to blob fields to find them fast */
     if (!(share->blob_field= save=
 	  (uint*) alloc_root(&outparam->mem_root,
                              (uint) (share->blob_fields* sizeof(uint)))))
       goto err;
-    for (i=0, ptr= outparam->field ; *ptr ; ptr++, i++)
+    for (k=0, ptr= outparam->field ; *ptr ; ptr++, k++)
     {
       if ((*ptr)->flags & BLOB_FLAG)
-	(*save++)= i;
+	(*save++)= k;
     }
   }
 
@@ -1432,7 +1446,19 @@ File create_frm(THD *thd, my_string name, const char *db,
     fileinfo[3]= (uchar) ha_checktype(thd,create_info->db_type,0,0);
     fileinfo[4]=1;
     int2store(fileinfo+6,IO_SIZE);		/* Next block starts here */
-    key_length=keys*(7+NAME_LEN+MAX_REF_PARTS*9)+16;
+    /*
+      Keep in sync with pack_keys() in unireg.cc
+      For each key:
+      8 bytes for the key header
+      9 bytes for each key-part (MAX_REF_PARTS)
+      NAME_LEN bytes for the name
+      1 byte for the NAMES_SEP_CHAR (before the name)
+      For all keys:
+      6 bytes for the header
+      1 byte for the NAMES_SEP_CHAR (after the last name)
+      9 extra bytes (padding for safety? alignment?)
+    */
+    key_length= keys * (8 + MAX_REF_PARTS * 9 + NAME_LEN + 1) + 16;
     length= next_io_size((ulong) (IO_SIZE+key_length+reclength+
                                   create_info->extra_size));
     int4store(fileinfo+10,length);
@@ -1587,7 +1613,7 @@ char *get_field(MEM_ROOT *mem, Field *field)
 
 bool check_db_name(char *name)
 {
-  char *start=name;
+  char *start= name;
   /* Used to catch empty names and names with end space */
   bool last_char_is_space= TRUE;
 
@@ -1758,11 +1784,11 @@ void st_table::reset_item_list(List<Item> *item_list) const
   calculate md5 of query
 
   SYNOPSIS
-    st_table_list::calc_md5()
+    TABLE_LIST::calc_md5()
     buffer	buffer for md5 writing
 */
 
-void  st_table_list::calc_md5(char *buffer)
+void  TABLE_LIST::calc_md5(char *buffer)
 {
   my_MD5_CTX context;
   uchar digest[16];
@@ -1787,10 +1813,10 @@ void  st_table_list::calc_md5(char *buffer)
     it (it is a kind of optimisation)
 
   SYNOPSIS
-    st_table_list::set_underlying_merge()
+    TABLE_LIST::set_underlying_merge()
 */
 
-void st_table_list::set_underlying_merge()
+void TABLE_LIST::set_underlying_merge()
 {
   TABLE_LIST *tbl;
 
@@ -1825,7 +1851,7 @@ void st_table_list::set_underlying_merge()
   setup fields of placeholder of merged VIEW
 
   SYNOPSIS
-    st_table_list::setup_underlying()
+    TABLE_LIST::setup_underlying()
     thd		    - thread handler
 
   DESCRIPTION
@@ -1838,9 +1864,9 @@ void st_table_list::set_underlying_merge()
     TRUE  - error
 */
 
-bool st_table_list::setup_underlying(THD *thd)
+bool TABLE_LIST::setup_underlying(THD *thd)
 {
-  DBUG_ENTER("st_table_list::setup_underlying");
+  DBUG_ENTER("TABLE_LIST::setup_underlying");
 
   if (!field_translation && merge_underlying_list)
   {
@@ -1903,7 +1929,7 @@ bool st_table_list::setup_underlying(THD *thd)
   Prepare where expression of view
 
   SYNOPSIS
-    st_table_list::prep_where()
+    TABLE_LIST::prep_where()
     thd             - thread handler
     conds           - condition of this JOIN
     no_where_clause - do not build WHERE or ON outer qwery do not need it
@@ -1917,10 +1943,10 @@ bool st_table_list::setup_underlying(THD *thd)
     TRUE  - error
 */
 
-bool st_table_list::prep_where(THD *thd, Item **conds,
+bool TABLE_LIST::prep_where(THD *thd, Item **conds,
                                bool no_where_clause)
 {
-  DBUG_ENTER("st_table_list::prep_where");
+  DBUG_ENTER("TABLE_LIST::prep_where");
 
   for (TABLE_LIST *tbl= merge_underlying_list; tbl; tbl= tbl->next_local)
   {
@@ -1958,12 +1984,13 @@ bool st_table_list::prep_where(THD *thd, Item **conds,
             this expression will not be moved to WHERE condition (i.e. will
             be clean correctly for PS/SP)
           */
-          tbl->on_expr= and_conds(tbl->on_expr, where);
+          tbl->on_expr= and_conds(tbl->on_expr,
+                                  where->copy_andor_structure(thd));
           break;
         }
       }
       if (tbl == 0)
-        *conds= and_conds(*conds, where);
+        *conds= and_conds(*conds, where->copy_andor_structure(thd));
       if (arena)
         thd->restore_active_arena(arena, &backup);
       where_processed= TRUE;
@@ -1974,11 +2001,52 @@ bool st_table_list::prep_where(THD *thd, Item **conds,
 }
 
 
+/*
+  Merge ON expressions for a view
+
+  SYNOPSIS
+    merge_on_conds()
+    thd             thread handle
+    table           table for the VIEW
+    is_cascaded     TRUE <=> merge ON expressions from underlying views
+
+  DESCRIPTION
+    This function returns the result of ANDing the ON expressions
+    of the given view and all underlying views. The ON expressions
+    of the underlying views are added only if is_cascaded is TRUE.
+
+  RETURN
+    Pointer to the built expression if there is any.
+    Otherwise and in the case of a failure NULL is returned.
+*/
+
+static Item *
+merge_on_conds(THD *thd, TABLE_LIST *table, bool is_cascaded)
+{
+  DBUG_ENTER("merge_on_conds");
+
+  Item *cond= NULL;
+  DBUG_PRINT("info", ("alias: %s", table->alias));
+  if (table->on_expr)
+    cond= table->on_expr->copy_andor_structure(thd);
+  if (!table->nested_join)
+    DBUG_RETURN(cond);
+  List_iterator<TABLE_LIST> li(table->nested_join->join_list);
+  while (TABLE_LIST *tbl= li++)
+  {
+    if (tbl->view && !is_cascaded)
+      continue;
+    cond= and_conds(cond, merge_on_conds(thd, tbl, is_cascaded));
+  }
+  DBUG_RETURN(cond);
+}
+
+
 /*
   Prepare check option expression of table
 
   SYNOPSIS
-    st_table_list::prep_check_option()
+    TABLE_LIST::prep_check_option()
     thd             - thread handler
     check_opt_type  - WITH CHECK OPTION type (VIEW_CHECK_NONE,
                       VIEW_CHECK_LOCAL, VIEW_CHECK_CASCADED)
@@ -1990,51 +2058,55 @@ bool st_table_list::prep_where(THD *thd, Item **conds,
                       VIEW_CHECK_LOCAL option.
 
   NOTE
-    This method build check options for every call
-    (usual execution or every SP/PS call)
+    This method builds check option condition to use it later on
+    every call (usual execution or every SP/PS call).
     This method have to be called after WHERE preparation
-    (st_table_list::prep_where)
+    (TABLE_LIST::prep_where)
 
   RETURN
     FALSE - OK
     TRUE  - error
 */
 
-bool st_table_list::prep_check_option(THD *thd, uint8 check_opt_type)
+bool TABLE_LIST::prep_check_option(THD *thd, uint8 check_opt_type)
 {
-  DBUG_ENTER("st_table_list::prep_check_option");
+  DBUG_ENTER("TABLE_LIST::prep_check_option");
+  bool is_cascaded= check_opt_type == VIEW_CHECK_CASCADED;
 
   for (TABLE_LIST *tbl= merge_underlying_list; tbl; tbl= tbl->next_local)
   {
     /* see comment of check_opt_type parameter */
-    if (tbl->view &&
-        tbl->prep_check_option(thd,
-                               ((check_opt_type == VIEW_CHECK_CASCADED) ?
-                                VIEW_CHECK_CASCADED :
-                                VIEW_CHECK_NONE)))
-    {
+    if (tbl->view && tbl->prep_check_option(thd, (is_cascaded ?
+                                                  VIEW_CHECK_CASCADED :
+                                                  VIEW_CHECK_NONE)))
       DBUG_RETURN(TRUE);
-    }
   }
 
-  if (check_opt_type)
+  if (check_opt_type && !check_option_processed)
   {
-    Item *item= 0;
+    Query_arena *arena= thd->stmt_arena, backup;
+    arena= thd->activate_stmt_arena_if_needed(&backup);  // For easier test
+
     if (where)
     {
       DBUG_ASSERT(where->fixed);
-      item= where->copy_andor_structure(thd);
+      check_option= where->copy_andor_structure(thd);
     }
-    if (check_opt_type == VIEW_CHECK_CASCADED)
+    if (is_cascaded)
     {
       for (TABLE_LIST *tbl= merge_underlying_list; tbl; tbl= tbl->next_local)
       {
         if (tbl->check_option)
-          item= and_conds(item, tbl->check_option);
+          check_option= and_conds(check_option, tbl->check_option);
       }
     }
-    if (item)
-      thd->change_item_tree(&check_option, item);
+    check_option= and_conds(check_option,
+                            merge_on_conds(thd, this, is_cascaded));
+
+    if (arena)
+      thd->restore_active_arena(arena, &backup);
+    check_option_processed= TRUE;
+
   }
 
   if (check_option)
@@ -2057,22 +2129,24 @@ bool st_table_list::prep_check_option(THD *thd, uint8 check_opt_type)
   Hide errors which show view underlying table information
 
   SYNOPSIS
-    st_table_list::hide_view_error()
+    TABLE_LIST::hide_view_error()
     thd     thread handler
 
 */
 
-void st_table_list::hide_view_error(THD *thd)
+void TABLE_LIST::hide_view_error(THD *thd)
 {
   /* Hide "Unknown column" or "Unknown function" error */
   if (thd->net.last_errno == ER_BAD_FIELD_ERROR ||
       thd->net.last_errno == ER_SP_DOES_NOT_EXIST ||
       thd->net.last_errno == ER_PROCACCESS_DENIED_ERROR ||
       thd->net.last_errno == ER_COLUMNACCESS_DENIED_ERROR ||
-      thd->net.last_errno == ER_TABLEACCESS_DENIED_ERROR)
+      thd->net.last_errno == ER_TABLEACCESS_DENIED_ERROR ||
+      thd->net.last_errno == ER_TABLE_NOT_LOCKED ||
+      thd->net.last_errno == ER_NO_SUCH_TABLE)
   {
     TABLE_LIST *top= top_table();
-    thd->clear_error();
+    thd->clear_error(); 
     my_error(ER_VIEW_INVALID, MYF(0), top->view_db.str, top->view_name.str);
   }
   else if (thd->net.last_errno == ER_NO_DEFAULT_FOR_FIELD)
@@ -2091,7 +2165,7 @@ void st_table_list::hide_view_error(THD *thd)
   table_to_find (TABLE)
 
   SYNOPSIS
-    st_table_list::find_underlying_table()
+    TABLE_LIST::find_underlying_table()
     table_to_find table to find
 
   RETURN
@@ -2099,7 +2173,7 @@ void st_table_list::hide_view_error(THD *thd)
     found table reference
 */
 
-st_table_list *st_table_list::find_underlying_table(TABLE *table_to_find)
+TABLE_LIST *TABLE_LIST::find_underlying_table(TABLE *table_to_find)
 {
   /* is this real table and table which we are looking for? */
   if (table == table_to_find && merge_underlying_list == 0)
@@ -2118,10 +2192,10 @@ st_table_list *st_table_list::find_underlying_table(TABLE *table_to_find)
   cleunup items belonged to view fields translation table
 
   SYNOPSIS
-    st_table_list::cleanup_items()
+    TABLE_LIST::cleanup_items()
 */
 
-void st_table_list::cleanup_items()
+void TABLE_LIST::cleanup_items()
 {
   if (!field_translation)
     return;
@@ -2137,7 +2211,7 @@ void st_table_list::cleanup_items()
   check CHECK OPTION condition
 
   SYNOPSIS
-    check_option()
+    TABLE_LIST::view_check_option()
     ignore_failure ignore check option fail
 
   RETURN
@@ -2146,23 +2220,21 @@ void st_table_list::cleanup_items()
     VIEW_CHECK_SKIP   FAILED, but continue
 */
 
-int st_table_list::view_check_option(THD *thd, bool ignore_failure)
+int TABLE_LIST::view_check_option(THD *thd, bool ignore_failure)
 {
   if (check_option && check_option->val_int() == 0)
   {
-    TABLE_LIST *view= top_table();
+    TABLE_LIST *main_view= top_table();
     if (ignore_failure)
     {
       push_warning_printf(thd, MYSQL_ERROR::WARN_LEVEL_ERROR,
                           ER_VIEW_CHECK_FAILED, ER(ER_VIEW_CHECK_FAILED),
-                          view->view_db.str, view->view_name.str);
+                          main_view->view_db.str, main_view->view_name.str);
       return(VIEW_CHECK_SKIP);
     }
-    else
-    {
-      my_error(ER_VIEW_CHECK_FAILED, MYF(0), view->view_db.str, view->view_name.str);
-      return(VIEW_CHECK_ERROR);
-    }
+    my_error(ER_VIEW_CHECK_FAILED, MYF(0), main_view->view_db.str,
+             main_view->view_name.str);
+    return(VIEW_CHECK_ERROR);
   }
   return(VIEW_CHECK_OK);
 }
@@ -2173,20 +2245,21 @@ int st_table_list::view_check_option(THD *thd, bool ignore_failure)
   table belong to given mask
 
   SYNOPSIS
-    st_table_list::check_single_table()
-    table	reference on variable where to store found table
+    TABLE_LIST::check_single_table()
+    table_arg	reference on variable where to store found table
 		(should be 0 on call, to find table, or point to table for
 		unique test)
     map         bit mask of tables
-    view        view for which we are looking table
+    view_arg    view for which we are looking table
 
   RETURN
     FALSE table not found or found only one
     TRUE  found several tables
 */
 
-bool st_table_list::check_single_table(st_table_list **table, table_map map,
-                                       st_table_list *view)
+bool TABLE_LIST::check_single_table(TABLE_LIST **table_arg,
+                                       table_map map,
+                                       TABLE_LIST *view_arg)
 {
   for (TABLE_LIST *tbl= merge_underlying_list; tbl; tbl= tbl->next_local)
   {
@@ -2194,13 +2267,13 @@ bool st_table_list::check_single_table(st_table_list **table, table_map map,
     {
       if (tbl->table->map & map)
       {
-	if (*table)
+	if (*table_arg)
 	  return TRUE;
-        *table= tbl;
-        tbl->check_option= view->check_option;
+        *table_arg= tbl;
+        tbl->check_option= view_arg->check_option;
       }
     }
-    else if (tbl->check_single_table(table, map, view))
+    else if (tbl->check_single_table(table_arg, map, view_arg))
       return TRUE;
   }
   return FALSE;
@@ -2219,7 +2292,7 @@ bool st_table_list::check_single_table(st_table_list **table, table_map map,
     TRUE  - out of memory
 */
 
-bool st_table_list::set_insert_values(MEM_ROOT *mem_root)
+bool TABLE_LIST::set_insert_values(MEM_ROOT *mem_root)
 {
   if (table)
   {
@@ -2243,7 +2316,7 @@ bool st_table_list::set_insert_values(MEM_ROOT *mem_root)
   Test if this is a leaf with respect to name resolution.
 
   SYNOPSIS
-    st_table_list::is_leaf_for_name_resolution()
+    TABLE_LIST::is_leaf_for_name_resolution()
 
   DESCRIPTION
     A table reference is a leaf with respect to name resolution if
@@ -2255,7 +2328,7 @@ bool st_table_list::set_insert_values(MEM_ROOT *mem_root)
   RETURN
     TRUE if a leaf, FALSE otherwise.
 */
-bool st_table_list::is_leaf_for_name_resolution()
+bool TABLE_LIST::is_leaf_for_name_resolution()
 {
   return (view || is_natural_join || is_join_columns_complete ||
           !nested_join);
@@ -2267,7 +2340,7 @@ bool st_table_list::is_leaf_for_name_resolution()
   respect to name resolution.
 
   SYNOPSIS
-    st_table_list::first_leaf_for_name_resolution()
+    TABLE_LIST::first_leaf_for_name_resolution()
 
   DESCRIPTION
     Given that 'this' is a nested table reference, recursively walk
@@ -2285,7 +2358,7 @@ bool st_table_list::is_leaf_for_name_resolution()
     else return 'this'
 */
 
-TABLE_LIST *st_table_list::first_leaf_for_name_resolution()
+TABLE_LIST *TABLE_LIST::first_leaf_for_name_resolution()
 {
   TABLE_LIST *cur_table_ref;
   NESTED_JOIN *cur_nested_join;
@@ -2325,7 +2398,7 @@ TABLE_LIST *st_table_list::first_leaf_for_name_resolution()
   respect to name resolution.
 
   SYNOPSIS
-    st_table_list::last_leaf_for_name_resolution()
+    TABLE_LIST::last_leaf_for_name_resolution()
 
   DESCRIPTION
     Given that 'this' is a nested table reference, recursively walk
@@ -2343,7 +2416,7 @@ TABLE_LIST *st_table_list::first_leaf_for_name_resolution()
     - else - 'this'
 */
 
-TABLE_LIST *st_table_list::last_leaf_for_name_resolution()
+TABLE_LIST *TABLE_LIST::last_leaf_for_name_resolution()
 {
   TABLE_LIST *cur_table_ref= this;
   NESTED_JOIN *cur_nested_join;
@@ -2385,7 +2458,7 @@ TABLE_LIST *st_table_list::last_leaf_for_name_resolution()
     want_access          Acess which we require
 */
 
-void st_table_list::register_want_access(ulong want_access)
+void TABLE_LIST::register_want_access(ulong want_access)
 {
   /* Remove SHOW_VIEW_ACL, because it will be checked during making view */
   want_access&= ~SHOW_VIEW_ACL;
@@ -2404,7 +2477,7 @@ void st_table_list::register_want_access(ulong want_access)
   Load security context information for this view
 
   SYNOPSIS
-    st_table_list::prepare_view_securety_context()
+    TABLE_LIST::prepare_view_securety_context()
     thd                  [in] thread handler
 
   RETURN
@@ -2413,9 +2486,9 @@ void st_table_list::register_want_access(ulong want_access)
 */
 
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
-bool st_table_list::prepare_view_securety_context(THD *thd)
+bool TABLE_LIST::prepare_view_securety_context(THD *thd)
 {
-  DBUG_ENTER("st_table_list::prepare_view_securety_context");
+  DBUG_ENTER("TABLE_LIST::prepare_view_securety_context");
   DBUG_PRINT("enter", ("table: %s", alias));
 
   DBUG_ASSERT(!prelocking_placeholder && view);
@@ -2429,7 +2502,8 @@ bool st_table_list::prepare_view_securety_context(THD *thd)
                                 definer.host.str,
                                 thd->db))
     {
-      if (thd->lex->sql_command == SQLCOM_SHOW_CREATE)
+      if ((thd->lex->sql_command == SQLCOM_SHOW_CREATE) ||
+          (thd->lex->sql_command == SQLCOM_SHOW_FIELDS))
       {
         push_warning_printf(thd, MYSQL_ERROR::WARN_LEVEL_NOTE, 
                             ER_NO_SUCH_USER, 
@@ -2438,7 +2512,18 @@ bool st_table_list::prepare_view_securety_context(THD *thd)
       }
       else
       {
-        my_error(ER_NO_SUCH_USER, MYF(0), definer.user.str, definer.host.str);
+        if (thd->security_ctx->master_access & SUPER_ACL)
+        {
+          my_error(ER_NO_SUCH_USER, MYF(0), definer.user.str, definer.host.str);
+
+        }
+        else
+        {
+           my_error(ER_ACCESS_DENIED_ERROR, MYF(0),
+                    thd->security_ctx->priv_user,
+                    thd->security_ctx->priv_host,
+                    (thd->password ?  ER(ER_YES) : ER(ER_NO)));
+        }
         DBUG_RETURN(TRUE);
       }
     }
@@ -2452,17 +2537,17 @@ bool st_table_list::prepare_view_securety_context(THD *thd)
   Find security context of current view
 
   SYNOPSIS
-    st_table_list::find_view_security_context()
+    TABLE_LIST::find_view_security_context()
     thd                  [in] thread handler
 
 */
 
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
-Security_context *st_table_list::find_view_security_context(THD *thd)
+Security_context *TABLE_LIST::find_view_security_context(THD *thd)
 {
   Security_context *sctx;
   TABLE_LIST *upper_view= this;
-  DBUG_ENTER("st_table_list::find_view_security_context");
+  DBUG_ENTER("TABLE_LIST::find_view_security_context");
 
   DBUG_ASSERT(view);
   while (upper_view && !upper_view->view_suid)
@@ -2491,7 +2576,7 @@ Security_context *st_table_list::find_view_security_context(THD *thd)
   Prepare security context and load underlying tables priveleges for view
 
   SYNOPSIS
-    st_table_list::prepare_security()
+    TABLE_LIST::prepare_security()
     thd                  [in] thread handler
 
   RETURN
@@ -2499,11 +2584,11 @@ Security_context *st_table_list::find_view_security_context(THD *thd)
     TRUE   Error
 */
 
-bool st_table_list::prepare_security(THD *thd)
+bool TABLE_LIST::prepare_security(THD *thd)
 {
   List_iterator_fast<TABLE_LIST> tb(*view_tables);
   TABLE_LIST *tbl;
-  DBUG_ENTER("st_table_list::prepare_security");
+  DBUG_ENTER("TABLE_LIST::prepare_security");
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   Security_context *save_security_ctx= thd->security_ctx;
 
@@ -2514,18 +2599,19 @@ bool st_table_list::prepare_security(THD *thd)
   while ((tbl= tb++))
   {
     DBUG_ASSERT(tbl->referencing_view);
-    char *db, *table_name;
+    char *local_db, *local_table_name;
     if (tbl->view)
     {
-      db= tbl->view_db.str;
-      table_name= tbl->view_name.str;
+      local_db= tbl->view_db.str;
+      local_table_name= tbl->view_name.str;
     }
     else
     {
-      db= tbl->db;
-      table_name= tbl->table_name;
+      local_db= tbl->db;
+      local_table_name= tbl->table_name;
     }
-    fill_effective_table_privileges(thd, &tbl->grant, db, table_name);
+    fill_effective_table_privileges(thd, &tbl->grant, local_db,
+                                    local_table_name);
     if (tbl->table)
       tbl->table->grant= grant;
   }
@@ -2597,6 +2683,7 @@ Field *Natural_join_column::field()
 
 const char *Natural_join_column::table_name()
 {
+  DBUG_ASSERT(table_ref);
   return table_ref->alias;
 }
 
@@ -2644,7 +2731,16 @@ const char *Field_iterator_table::name()
 
 Item *Field_iterator_table::create_item(THD *thd)
 {
-  return new Item_field(thd, &thd->lex->current_select->context, *ptr);
+  SELECT_LEX *select= thd->lex->current_select;
+
+  Item_field *item= new Item_field(thd, &select->context, *ptr);
+  if (item && thd->variables.sql_mode & MODE_ONLY_FULL_GROUP_BY &&
+      !thd->lex->in_sum_func && select->cur_pos_in_select_list != UNDEF_POS)
+  {
+    select->non_agg_fields.push_back(item);
+    item->marker= select->cur_pos_in_select_list;
+  }
+  return item;
 }
 
 
@@ -2884,12 +2980,13 @@ Field_iterator_table_ref::get_or_create_column_ref(TABLE_LIST *parent_table_ref)
   uint field_count;
   TABLE_LIST *add_table_ref= parent_table_ref ?
                              parent_table_ref : table_ref;
-
+  LINT_INIT(field_count);
+  
   if (field_it == &table_field_it)
   {
     /* The field belongs to a stored table. */
-    Field *field= table_field_it.field();
-    nj_col= new Natural_join_column(field, table_ref);
+    Field *tmp_field= table_field_it.field();
+    nj_col= new Natural_join_column(tmp_field, table_ref);
     field_count= table_ref->table->s->fields;
   }
   else if (field_it == &view_field_it)
@@ -2982,6 +3079,53 @@ Field_iterator_table_ref::get_natural_column_ref()
   return nj_col;
 }
 
+/*
+  Cleanup this table for re-execution.
+
+  SYNOPSIS
+    TABLE_LIST::reinit_before_use()
+*/
+
+void TABLE_LIST::reinit_before_use(THD *thd)
+{
+  /*
+    Reset old pointers to TABLEs: they are not valid since the tables
+    were closed in the end of previous prepare or execute call.
+  */
+  table= 0;
+  /* Reset is_schema_table_processed value(needed for I_S tables */
+  schema_table_state= NOT_PROCESSED;
+
+  TABLE_LIST *embedded; /* The table at the current level of nesting. */
+  TABLE_LIST *parent_embedding= this; /* The parent nested table reference. */
+  do
+  {
+    embedded= parent_embedding;
+    if (embedded->prep_on_expr)
+      embedded->on_expr= embedded->prep_on_expr->copy_andor_structure(thd);
+    parent_embedding= embedded->embedding;
+  }
+  while (parent_embedding &&
+         parent_embedding->nested_join->join_list.head() == embedded);
+}
+
+/*
+  Return subselect that contains the FROM list this table is taken from
+
+  SYNOPSIS
+    TABLE_LIST::containing_subselect()
+ 
+  RETURN
+    Subselect item for the subquery that contains the FROM list
+    this table is taken from if there is any
+    0 - otherwise
+
+*/
+
+Item_subselect *TABLE_LIST::containing_subselect()
+{    
+  return (select_lex ? select_lex->master_unit()->item : 0);
+}
 
 /*****************************************************************************
 ** Instansiate templates
