@@ -55,9 +55,21 @@ typedef struct st_grant_info
   ulong orig_want_privilege;
 } GRANT_INFO;
 
-enum tmp_table_type {NO_TMP_TABLE=0, TMP_TABLE=1, TRANSACTIONAL_TMP_TABLE=2,
+enum tmp_table_type {NO_TMP_TABLE=0,
+                     NON_TRANSACTIONAL_TMP_TABLE=1, TRANSACTIONAL_TMP_TABLE=2,
                      SYSTEM_TMP_TABLE=3};
 
+
+/** Event on which trigger is invoked. */
+enum trg_event_type
+{
+  TRG_EVENT_INSERT= 0,
+  TRG_EVENT_UPDATE= 1,
+  TRG_EVENT_DELETE= 2,
+  TRG_EVENT_MAX
+};
+
+
 enum frm_type_enum
 {
   FRMTYPE_ERROR= 0,
@@ -136,7 +148,12 @@ typedef struct st_table_share
   const char *table_name;               /* Table name (for open) */
   const char *path;                     /* Path to .frm file (from datadir) */
   LEX_STRING connect_string;
-  key_map keys_in_use;                  /* Keys in use for table */
+
+  /* 
+     Set of keys in use, implemented as a Bitmap.
+     Excludes keys disabled by ALTER TABLE ... DISABLE KEYS.
+  */
+  key_map keys_in_use;
   key_map keys_for_keyread;
   ulong   avg_row_length;		/* create information */
   ulong   raid_chunksize;
@@ -187,6 +204,8 @@ typedef struct st_table_share
 } TABLE_SHARE;
 
 
+extern ulong refresh_version;
+
 /* Information for one open table */
 
 struct st_table {
@@ -205,7 +224,21 @@ struct st_table {
 
   byte *record[2];			/* Pointer to records */
   byte *insert_values;                  /* used by INSERT ... UPDATE */
-  key_map quick_keys, used_keys, keys_in_use_for_query;
+  key_map quick_keys, used_keys;
+
+  /*
+    A set of keys that can be used in the query that references this
+    table 
+
+    All indexes disabled on the table's TABLE_SHARE (see TABLE::s) will be 
+    subtracted from this set upon instantiation. Thus for any TABLE t it holds
+    that t.keys_in_use_for_query is a subset of t.s.keys_in_use. Generally we 
+    must not introduce any new keys here (see setup_tables).
+
+    The set is implemented as a bitmap.
+  */
+  key_map keys_in_use_for_query;
+  key_map merge_keys;
   KEY  *key_info;			/* data of keys in database */
 
   Field *next_number_field,		/* Set if next_number is activated */
@@ -215,7 +248,7 @@ struct st_table {
 
   /* Table's triggers, 0 if there are no of them */
   Table_triggers_list *triggers;
-  struct st_table_list *pos_in_table_list;/* Element referring to this table */
+  TABLE_LIST *pos_in_table_list;/* Element referring to this table */
   ORDER		*group;
   const char	*alias;            	  /* alias or table name */
   uchar		*null_flags;
@@ -267,12 +300,34 @@ struct st_table {
   my_bool force_index;
   my_bool distinct,const_table,no_rows;
   my_bool key_read, no_keyread;
-  my_bool locked_by_flush;
+  /*
+    Placeholder for an open table which prevents other connections
+    from taking name-locks on this table. Typically used with
+    TABLE_SHARE::version member to take an exclusive name-lock on
+    this table name -- a name lock that not only prevents other
+    threads from opening the table, but also blocks other name
+    locks. This is achieved by:
+    - setting open_placeholder to 1 - this will block other name
+      locks, as wait_for_locked_table_name will be forced to wait,
+      see table_is_used for details.
+    - setting version to 0 - this will force other threads to close
+      the instance of this table and wait (this is the same approach
+      as used for usual name locks).
+    An exclusively name-locked table currently can have no handler
+    object associated with it (db_stat is always 0), but please do
+    not rely on that.
+  */
+  my_bool open_placeholder;
   my_bool locked_by_name;
   my_bool fulltext_searched;
   my_bool no_cache;
   /* To signal that we should reset query_id for tables and cols */
   my_bool clear_query_id;
+  /*
+    To indicate that a non-null value of the auto_increment field
+    was provided by the user or retrieved from the current record.
+    Used only in the MODE_NO_AUTO_VALUE_ON_ZERO mode.
+  */
   my_bool auto_increment_field_not_null;
   my_bool insert_or_update;             /* Can be used by the handler */
   my_bool alias_name_used;		/* true if table_name is alias */
@@ -285,6 +340,13 @@ struct st_table {
 
   bool fill_item_list(List<Item> *item_list) const;
   void reset_item_list(List<Item> *item_list) const;
+  /* Is table open or should be treated as such by name-locking? */
+  inline bool is_name_opened() { return db_stat || open_placeholder; }
+  /*
+    Is this instance of the table should be reopen or represents a name-lock?
+  */
+  inline bool needs_reopen_or_name_lock()
+  { return s->version != refresh_version; }
 };
 
 enum enum_schema_table_state
@@ -314,7 +376,7 @@ enum enum_schema_tables
   SCH_COLUMN_PRIVILEGES,
   SCH_KEY_COLUMN_USAGE,
   SCH_OPEN_TABLES,
-  SCH_PROFILING,
+  SCH_PROFILES,
   SCH_PROCEDURES,
   SCH_SCHEMATA,
   SCH_SCHEMA_PRIVILEGES,
@@ -342,7 +404,7 @@ typedef struct st_field_info
 } ST_FIELD_INFO;
 
 
-struct st_table_list;
+struct TABLE_LIST;
 typedef class Item COND;
 
 typedef struct st_schema_table
@@ -350,12 +412,12 @@ typedef struct st_schema_table
   const char* table_name;
   ST_FIELD_INFO *fields_info;
   /* Create information_schema table */
-  TABLE *(*create_table)  (THD *thd, struct st_table_list *table_list);
+  TABLE *(*create_table)  (THD *thd, TABLE_LIST *table_list);
   /* Fill table with data */
-  int (*fill_table) (THD *thd, struct st_table_list *tables, COND *cond);
+  int (*fill_table) (THD *thd, TABLE_LIST *tables, COND *cond);
   /* Handle fileds for old SHOW */
   int (*old_format) (THD *thd, struct st_schema_table *schema_table);
-  int (*process_table) (THD *thd, struct st_table_list *tables,
+  int (*process_table) (THD *thd, TABLE_LIST *tables,
                         TABLE *table, bool res, const char *base_name,
                         const char *file_name);
   int idx_field1, idx_field2; 
@@ -388,7 +450,7 @@ struct st_lex;
 class select_union;
 class TMP_TABLE_PARAM;
 
-Item *create_view_field(THD *thd, st_table_list *view, Item **field_ref,
+Item *create_view_field(THD *thd, TABLE_LIST *view, Item **field_ref,
                         const char *name);
 
 struct Field_translator
@@ -409,7 +471,7 @@ class Natural_join_column: public Sql_alloc
 public:
   Field_translator *view_field;  /* Column reference of merge view. */
   Field            *table_field; /* Column reference of table or temp view. */
-  st_table_list *table_ref; /* Original base table/view reference. */
+  TABLE_LIST *table_ref; /* Original base table/view reference. */
   /*
     True if a common join column of two NATURAL/USING join operands. Notice
     that when we have a hierarchy of nested NATURAL/USING joins, a column can
@@ -419,8 +481,8 @@ public:
   */
   bool is_common;
 public:
-  Natural_join_column(Field_translator *field_param, st_table_list *tab);
-  Natural_join_column(Field *field_param, st_table_list *tab);
+  Natural_join_column(Field_translator *field_param, TABLE_LIST *tab);
+  Natural_join_column(Field *field_param, TABLE_LIST *tab);
   const char *name();
   Item *create_item(THD *thd);
   Field *field();
@@ -462,17 +524,17 @@ public:
          (TABLE_LIST::join_using_fields != NULL)
 */
 
-typedef struct st_table_list
+struct TABLE_LIST
 {
-  st_table_list() {}                          /* Remove gcc warning */
+  TABLE_LIST() {}                          /* Remove gcc warning */
   /*
     List of tables local to a subquery (used by SQL_LIST). Considers
     views as leaves (unlike 'next_leaf' below). Created at parse time
     in st_select_lex::add_table_to_list() -> table_list.link_in_list().
   */
-  struct st_table_list *next_local;
+  TABLE_LIST *next_local;
   /* link in a global list of all queries tables */
-  struct st_table_list *next_global, **prev_global;
+  TABLE_LIST *next_global, **prev_global;
   char		*db, *alias, *table_name, *schema_table_name;
   char          *option;                /* Used by cache index  */
   Item		*on_expr;		/* Used with outer join */
@@ -492,7 +554,7 @@ typedef struct st_table_list
     'this' represents a NATURAL or USING join operation. Thus after
     parsing 'this' is a NATURAL/USING join iff (natural_join != NULL).
   */
-  struct st_table_list *natural_join;
+  TABLE_LIST *natural_join;
   /*
     True if 'this' represents a nested join that is a NATURAL JOIN.
     For one of the operands of 'this', the member 'natural_join' points
@@ -516,7 +578,7 @@ typedef struct st_table_list
     base tables. All of these TABLE_LIST instances contain a
     materialized list of columns. The list is local to a subquery.
   */
-  struct st_table_list *next_name_resolution_table;
+  TABLE_LIST *next_name_resolution_table;
   /* Index names in a "... JOIN ... USE/IGNORE INDEX ..." clause. */
   List<String> *use_index, *ignore_index;
   TABLE        *table;                   /* opened table */
@@ -532,7 +594,7 @@ typedef struct st_table_list
     here it will be reference of first occurrence of t1 to second (as you
     can see this lists can't be merged)
   */
-  st_table_list	*correspondent_table;
+  TABLE_LIST	*correspondent_table;
   st_select_lex_unit *derived;		/* SELECT_LEX_UNIT of derived table */
   ST_SCHEMA_TABLE *schema_table;        /* Information_schema table */
   st_select_lex	*schema_select_lex;
@@ -553,20 +615,20 @@ typedef struct st_table_list
     does not include the tables of subqueries used in the view. Is set only
     for merged views.
   */
-  st_table_list	*merge_underlying_list;
+  TABLE_LIST	*merge_underlying_list;
   /*
     - 0 for base tables
     - in case of the view it is the list of all (not only underlying
     tables but also used in subquery ones) tables of the view.
   */
-  List<st_table_list> *view_tables;
+  List<TABLE_LIST> *view_tables;
   /* most upper view this table belongs to */
-  st_table_list	*belong_to_view;
+  TABLE_LIST	*belong_to_view;
   /*
     The view directly referencing this table
     (non-zero only for merged underlying tables of a view).
   */
-  st_table_list	*referencing_view;
+  TABLE_LIST	*referencing_view;
   /*
     Security  context (non-zero only for tables which belong
     to view with SQL SECURITY DEFINER)
@@ -583,7 +645,7 @@ typedef struct st_table_list
     leaves. Created in setup_tables() -> make_leaves_list().
   */
   bool allowed_show;
-  st_table_list	*next_leaf;
+  TABLE_LIST	*next_leaf;
   Item          *where;                 /* VIEW WHERE clause condition */
   Item          *check_option;          /* WITH CHECK OPTION condition */
   LEX_STRING	query;			/* text of (CRETE/SELECT) statement */
@@ -623,8 +685,8 @@ typedef struct st_table_list
   table_map     dep_tables;             /* tables the table depends on      */
   table_map     on_expr_dep_tables;     /* tables on expression depends on  */
   struct st_nested_join *nested_join;   /* if the element is a nested join  */
-  st_table_list *embedding;             /* nested join containing the table */
-  List<struct st_table_list> *join_list;/* join list the table belongs to   */
+  TABLE_LIST *embedding;             /* nested join containing the table */
+  List<TABLE_LIST> *join_list;/* join list the table belongs to   */
   bool		cacheable_table;	/* stop PS caching */
   /* used in multi-upd/views privilege check */
   bool		table_in_first_from_clause;
@@ -635,6 +697,8 @@ typedef struct st_table_list
   bool          compact_view_format;    /* Use compact format for SHOW CREATE VIEW */
   /* view where processed */
   bool          where_processed;
+  /* TRUE <=> VIEW CHECK OPTION expression has been processed */
+  bool          check_option_processed;
   /* FRMTYPE_ERROR if any type is acceptable */
   enum frm_type_enum required_type;
   char		timestamp_buffer[20];	/* buffer for timestamp (19+1) */
@@ -643,6 +707,19 @@ typedef struct st_table_list
     used for implicit LOCK TABLES only and won't be used in real statement.
   */
   bool          prelocking_placeholder;
+  /*
+    This TABLE_LIST object corresponds to the table to be created
+    so it is possible that it does not exist (used in CREATE TABLE
+    ... SELECT implementation).
+  */
+  bool          create;
+
+  /**
+    Indicates what triggers we need to pre-load for this TABLE_LIST
+    when opening an associated TABLE. This is filled after
+    the parsed tree is created.
+  */
+  uint8 trg_event_map;
 
   enum enum_schema_table_state schema_table_state;
   void calc_md5(char *buffer);
@@ -650,17 +727,21 @@ typedef struct st_table_list
   int view_check_option(THD *thd, bool ignore_failure);
   bool setup_underlying(THD *thd);
   void cleanup_items();
-  bool placeholder() {return derived || view; }
+  bool placeholder()
+  {
+    return derived || view || schema_table || create && !table->db_stat ||
+           !table;
+  }
   void print(THD *thd, String *str);
-  bool check_single_table(st_table_list **table, table_map map,
-                          st_table_list *view);
+  bool check_single_table(TABLE_LIST **table, table_map map,
+                          TABLE_LIST *view);
   bool set_insert_values(MEM_ROOT *mem_root);
   void hide_view_error(THD *thd);
-  st_table_list *find_underlying_table(TABLE *table);
-  st_table_list *first_leaf_for_name_resolution();
-  st_table_list *last_leaf_for_name_resolution();
+  TABLE_LIST *find_underlying_table(TABLE *table);
+  TABLE_LIST *first_leaf_for_name_resolution();
+  TABLE_LIST *last_leaf_for_name_resolution();
   bool is_leaf_for_name_resolution();
-  inline st_table_list *top_table()
+  inline TABLE_LIST *top_table()
     { return belong_to_view ? belong_to_view : this; }
   inline bool prepare_check_option(THD *thd)
   {
@@ -697,7 +778,7 @@ private:
     Cleanup for re-execution in a prepared statement or a stored
     procedure.
   */
-} TABLE_LIST;
+};
 
 class Item;
 
