@@ -2,8 +2,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,8 +42,6 @@
 
 #if defined(HAVE_LONG_LONG) && !defined(longlong2str) && !defined(HAVE_LONGLONG2STR)
 
-extern char NEAR _dig_vec[];
-
 /*
   This assumes that longlong multiplication is faster than longlong division.
 */
@@ -81,14 +78,14 @@ char *longlong2str(longlong val,char *dst,int radix)
   {
     ulonglong quo=(ulonglong) val/(uint) radix;
     uint rem= (uint) (val- quo* (uint) radix);
-    *--p = _dig_vec[rem];
+    *--p = _dig_vec_upper[rem];
     val= quo;
   }
   long_val= (long) val;
   while (long_val != 0)
   {
     long quo= long_val/radix;
-    *--p = _dig_vec[(uchar) (long_val - quo*radix)];
+    *--p = _dig_vec_upper[(uchar) (long_val - quo*radix)];
     long_val= quo;
   }
   while ((*dst++ = *p++) != 0) ;
@@ -126,14 +123,14 @@ char *longlong10_to_str(longlong val,char *dst,int radix)
   {
     ulonglong quo=(ulonglong) val/(uint) 10;
     uint rem= (uint) (val- quo* (uint) 10);
-    *--p = _dig_vec[rem];
+    *--p = _dig_vec_upper[rem];
     val= quo;
   }
   long_val= (long) val;
   while (long_val != 0)
   {
     long quo= long_val/10;
-    *--p = _dig_vec[(uchar) (long_val - quo*10)];
+    *--p = _dig_vec_upper[(uchar) (long_val - quo*10)];
     long_val= quo;
   }
   while ((*dst++ = *p++) != 0) ;
